@@ -29,7 +29,8 @@ DIR_LABS="/c/Graphviz"
 
 # needed to find headers and libs at compile time. Must use absolute
 # Windows path for libs (why?)
-export CFLAGS="-I$DIR_LABS/include -L$DIR_WABS/lib"
+export CFLAGS="-I$DIR_LABS/include"
+export LDFLAGS="-L$DIR_LABS/lib"
 
 # needed to find e.g. libgvc.dll at run time. Windows does not use
 # LD_LIBRARY_PATH. Must be the logical directory
@@ -38,4 +39,4 @@ export PATH="${PATH}:$DIR_LABS/bin"
 python gen_version.py --output GRAPHVIZ_VERSION
 export GV_VERSION=$( cat GRAPHVIZ_VERSION )
 
-python3 -m pytest --verbose ci/tests.py tests
+python3 -m pytest --strict-markers --verbose --verbose ci/tests.py tests
