@@ -1,9 +1,10 @@
+/// @file
+/// @ingroup cgraph_utils
 #pragma once
 
 #include <assert.h>
 #include <cgraph/alloc.h>
 #include <cgraph/exit.h>
-#include <cgraph/likely.h>
 #include <errno.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -22,7 +23,7 @@
  * \param name Type name to give the list container
  * \param type Type of the elements the list will store
  */
-#define DEFINE_LIST(name, type) DEFINE_LIST_WITH_DTOR(name, type, NULL)
+#define DEFINE_LIST(name, type) DEFINE_LIST_WITH_DTOR(name, type, name##_noop_)
 
 /** \p DEFINE_LIST but with a custom element destructor
  *
@@ -43,6 +44,9 @@
     size_t capacity; /* available storage slots */                             \
   } name##_t;                                                                  \
                                                                                \
+  /* default “do nothing” destructor */                                    \
+  static inline LIST_UNUSED void name##_noop_(type item) { (void)item; }       \
+                                                                               \
   /** get the number of elements in a list */                                  \
   static inline LIST_UNUSED size_t name##_size(const name##_t *list) {         \
     assert(list != NULL);                                                      \
@@ -63,12 +67,12 @@
       const size_t c = list->capacity == 0 ? 1 : (list->capacity * 2);         \
                                                                                \
       /* will the calculation of the new size overflow? */                     \
-      if (UNLIKELY(SIZE_MAX / c < sizeof(type))) {                             \
+      if (SIZE_MAX / c < sizeof(type)) {                                       \
         return ERANGE;                                                         \
       }                                                                        \
                                                                                \
       type *data = (type *)realloc(list->data, c * sizeof(type));              \
-      if (UNLIKELY(data == NULL)) {                                            \
+      if (data == NULL) {                                                      \
         return ENOMEM;                                                         \
       }                                                                        \
                                                                                \
@@ -100,28 +104,56 @@
    * \param index Element index to get                                         \
    * \return Element at the given index                                        \
    */                                                                          \
-  static inline LIST_UNUSED type name##_get(const name##_t *list,              \
-                                            size_t index) {                    \
+  static inline type name##_get(const name##_t *list, size_t index) {          \
     assert(list != NULL);                                                      \
     assert(index < list->size && "index out of bounds");                       \
     return list->data[index];                                                  \
   }                                                                            \
                                                                                \
+  /** access an element in a list for the purpose of modification              \
+   *                                                                           \
+   * Because this acquires an internal pointer into the list structure, `get`  \
+   * and `set` should be preferred over this function. `get` and `set` are     \
+   * easier to reason about. In particular, the pointer returned by this       \
+   * function is invalidated by any list operation that may reallocate the     \
+   * backing storage (e.g. `shrink_to_fit`).                                   \
+   *                                                                           \
+   * \param list List to operate on                                            \
+   * \param index Element to get a pointer to                                  \
+   * \return Pointer to the requested element                                  \
+   */                                                                          \
+  static inline type *name##_at(name##_t *list, size_t index) {                \
+    assert(list != NULL);                                                      \
+    assert(index < list->size && "index out of bounds");                       \
+    return &list->data[index];                                                 \
+  }                                                                            \
+                                                                               \
+  /** get a handle to the first element */                                     \
+  static inline LIST_UNUSED type *name##_front(name##_t *list) {               \
+    assert(list != NULL);                                                      \
+    assert(!name##_is_empty(list));                                            \
+    return name##_at(list, 0);                                                 \
+  }                                                                            \
+                                                                               \
+  /** get a handle to the last element */                                      \
+  static inline LIST_UNUSED type *name##_back(name##_t *list) {                \
+    assert(list != NULL);                                                      \
+    assert(!name##_is_empty(list));                                            \
+    return name##_at(list, name##_size(list) - 1);                             \
+  }                                                                            \
+                                                                               \
   /** assign to an element in a list                                           \
    *                                                                           \
    * \param list List to operate on                                            \
    * \param index Element to assign to                                         \
    * \param item Value to assign                                               \
    */                                                                          \
-  static inline LIST_UNUSED void name##_set(name##_t *list, size_t index,      \
-                                            type item) {                       \
+  static inline void name##_set(name##_t *list, size_t index, type item) {     \
     assert(list != NULL);                                                      \
-    assert(index < list->size && "index out of bounds");                       \
-    void (*dtor_)(type) = (void (*)(type))(dtor);                              \
-    if (dtor_ != NULL) {                                                       \
-      dtor_(list->data[index]);                                                \
-    }                                                                          \
-    list->data[index] = item;                                                  \
+    assert(index < name##_size(list) && "index out of bounds");                \
+    type *target = name##_at(list, index);                                     \
+    dtor(*target);                                                             \
+    *target = item;                                                            \
   }                                                                            \
                                                                                \
   /** remove an element from a list                                            \
@@ -138,10 +170,7 @@
       if (memcmp(&list->data[i], &item, sizeof(type)) == 0) {                  \
                                                                                \
         /* destroy the element we are about to remove */                       \
-        void (*dtor_)(type) = (void (*)(type))(dtor);                          \
-        if (dtor_ != NULL) {                                                   \
-          dtor_(list->data[i]);                                                \
-        }                                                                      \
+        dtor(list->data[i]);                                                   \
                                                                                \
         /* shrink the list */                                                  \
         size_t remainder = (list->size - i - 1) * sizeof(type);                \
@@ -152,33 +181,12 @@
     }                                                                          \
   }                                                                            \
                                                                                \
-  /** access an element in a list for the purpose of modification              \
-   *                                                                           \
-   * Because this acquires an internal pointer into the list structure, `get`  \
-   * and `set` should be preferred over this function. `get` and `set` are     \
-   * easier to reason about. In particular, the pointer returned by this       \
-   * function is invalidated by any list operation that may reallocate the     \
-   * backing storage (e.g. `shrink_to_fit`).                                   \
-   *                                                                           \
-   * \param list List to operate on                                            \
-   * \param index Element to get a pointer to                                  \
-   * \return Pointer to the requested element                                  \
-   */                                                                          \
-  static inline LIST_UNUSED type *name##_at(name##_t *list, size_t index) {    \
-    assert(list != NULL);                                                      \
-    assert(index < list->size && "index out of bounds");                       \
-    return &list->data[index];                                                 \
-  }                                                                            \
-                                                                               \
   /** remove all elements from a list */                                       \
   static inline LIST_UNUSED void name##_clear(name##_t *list) {                \
     assert(list != NULL);                                                      \
                                                                                \
-    void (*dtor_)(type) = (void (*)(type))(dtor);                              \
-    if (dtor_ != NULL) {                                                       \
-      for (size_t i = 0; i < list->size; ++i) {                                \
-        dtor_(list->data[i]);                                                  \
-      }                                                                        \
+    for (size_t i = 0; i < list->size; ++i) {                                  \
+      dtor(list->data[i]);                                                     \
     }                                                                          \
                                                                                \
     list->size = 0;                                                            \
@@ -221,15 +229,37 @@
     } else if (list->size > size) {                                            \
       /* we are shrinking the list */                                          \
       while (list->size > size) {                                              \
-        void (*dtor_)(type) = (void (*)(type))(dtor);                          \
-        if (dtor_ != NULL) {                                                   \
-          dtor_(list->data[list->size - 1]);                                   \
-        }                                                                      \
+        dtor(list->data[list->size - 1]);                                      \
         --list->size;                                                          \
       }                                                                        \
     }                                                                          \
   }                                                                            \
                                                                                \
+  /** is the given element in the list? */                                     \
+  static inline LIST_UNUSED bool name##_contains(                              \
+      const name##_t *haystack, const type needle,                             \
+      bool (*eq)(const type a, const type b)) {                                \
+    assert(haystack != NULL);                                                  \
+    assert(eq != NULL);                                                        \
+                                                                               \
+    for (size_t i = 0; i < name##_size(haystack); ++i) {                       \
+      if (eq(name##_get(haystack, i), needle)) {                               \
+        return true;                                                           \
+      }                                                                        \
+    }                                                                          \
+    return false;                                                              \
+  }                                                                            \
+                                                                               \
+  /** replicate a list */                                                      \
+  static inline LIST_UNUSED name##_t name##_copy(const name##_t *source) {     \
+    assert(source != NULL);                                                    \
+                                                                               \
+    name##_t destination = {(type *)gv_calloc(source->capacity, sizeof(type)), \
+                            source->size, source->capacity};                   \
+    memcpy(destination.data, source->data, source->size * sizeof(type));       \
+    return destination;                                                        \
+  }                                                                            \
+                                                                               \
   /** sort the list using the given comparator */                              \
   static inline LIST_UNUSED void name##_sort(                                  \
       name##_t *list, int (*cmp)(const type *a, const type *b)) {              \
@@ -243,6 +273,18 @@
     }                                                                          \
   }                                                                            \
                                                                                \
+  /** flip the order of elements in the list */                                \
+  static inline LIST_UNUSED void name##_reverse(name##_t *list) {              \
+    assert(list != NULL);                                                      \
+                                                                               \
+    for (size_t i = 0; i < name##_size(list) / 2; ++i) {                       \
+      type const temp1 = name##_get(list, i);                                  \
+      type const temp2 = name##_get(list, name##_size(list) - i - 1);          \
+      name##_set(list, i, temp2);                                              \
+      name##_set(list, name##_size(list) - i - 1, temp1);                      \
+    }                                                                          \
+  }                                                                            \
+                                                                               \
   /** deallocate unused backing storage, shrinking capacity to size */         \
   static inline LIST_UNUSED void name##_shrink_to_fit(name##_t *list) {        \
     assert(list != NULL);                                                      \
@@ -263,12 +305,13 @@
   }                                                                            \
                                                                                \
   /** alias for append */                                                      \
-  static inline LIST_UNUSED void name##_push(name##_t *list, type value) {     \
+  static inline LIST_UNUSED void name##_push_back(name##_t *list,              \
+                                                  type value) {                \
     name##_append(list, value);                                                \
   }                                                                            \
                                                                                \
   /** remove and return last element */                                        \
-  static inline LIST_UNUSED type name##_pop(name##_t *list) {                  \
+  static inline LIST_UNUSED type name##_pop_back(name##_t *list) {             \
     assert(list != NULL);                                                      \
     assert(list->size > 0);                                                    \
                                                                                \
