@@ -1,18 +1,23 @@
+/**
+ * @file
+ * @ingroup cgraph_core
+ * @ingroup cgraph_rec
+ */
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include	<cgraph/cghdr.h>
-#include	<cgraph/streq.h>
 #include	<cgraph/unreachable.h>
 #include	<stdbool.h>
 #include	<stddef.h>
+#include	<util/streq.h>
 
 /*
  * run time records
@@ -49,7 +54,7 @@ Agrec_t *aggetrec(void *obj, const char *name, int mtf)
 
     if (hdr->tag.mtflock) {
 	if (mtf && hdr->data != d)
-	    agerr(AGERR, "move to front lock inconsistency");
+	    agerrorf("move to front lock inconsistency");
     } else if (d != first || mtf != 0) {
 	set_data(hdr, d, mtf != 0);	/* Always optimize */
     }
