@@ -14,7 +14,22 @@ DEFINE_LIST(ints, int)
 
 // test construction and destruction, with nothing in-between
 static void test_create_reset(void) {
-  ints_t i = ints_new();
+  ints_t i = {0};
+  ints_free(&i);
+}
+
+// a list should start in a known initial state
+static void test_init(void) {
+  ints_t i = {0};
+  assert(ints_is_empty(&i));
+  assert(ints_size(&i) == 0);
+}
+
+// reset of an initialized list should be OK and idempotent
+static void test_init_reset(void) {
+  ints_t i = {0};
+  ints_free(&i);
+  ints_free(&i);
   ints_free(&i);
 }
 
@@ -175,6 +190,59 @@ static void test_clear(void) {
   ints_free(&xs);
 }
 
+// basic push then pop
+static void test_push_one(void) {
+  ints_t s = {0};
+  int arbitrary = 42;
+  ints_push_back(&s, arbitrary);
+  assert(ints_size(&s) == 1);
+  int top = ints_pop_back(&s);
+  assert(top == arbitrary);
+  assert(ints_is_empty(&s));
+  ints_free(&s);
+}
+
+static void push_then_pop(int count) {
+  ints_t s = {0};
+  for (int i = 0; i < count; ++i) {
+    ints_push_back(&s, i);
+    assert(ints_size(&s) == (size_t)i + 1);
+  }
+  for (int i = count - 1;; --i) {
+    assert(ints_size(&s) == (size_t)i + 1);
+    int p = ints_pop_back(&s);
+    assert(p == i);
+    if (i == 0) {
+      break;
+    }
+  }
+  ints_free(&s);
+}
+
+// push a series of items
+static void test_push_then_pop_ten(void) { push_then_pop(10); }
+
+// push enough to cause an expansion
+static void test_push_then_pop_many(void) { push_then_pop(4096); }
+
+// interleave some push and pop operations
+static void test_push_pop_interleaved(void) {
+  ints_t s = {0};
+  size_t size = 0;
+  for (int i = 0; i < 4096; ++i) {
+    if (i % 3 == 1) {
+      int p = ints_pop_back(&s);
+      assert(p == i - 1);
+      --size;
+    } else {
+      ints_push_back(&s, i);
+      ++size;
+    }
+    assert(ints_size(&s) == size);
+  }
+  ints_free(&s);
+}
+
 static void test_resize_empty_1(void) {
   ints_t xs = {0};
   ints_resize(&xs, 0, 0);
@@ -350,13 +418,13 @@ static void test_free(void) {
   assert(xs.capacity == 0);
 }
 
-static void test_push(void) {
+static void test_push_back(void) {
   ints_t xs = {0};
   ints_t ys = {0};
 
   for (size_t i = 0; i < 10; ++i) {
     ints_append(&xs, (int)i);
-    ints_push(&ys, (int)i);
+    ints_push_back(&ys, (int)i);
     assert(ints_size(&xs) == ints_size(&ys));
     for (size_t j = 0; j <= i; ++j) {
       assert(ints_get(&xs, j) == ints_get(&ys, j));
@@ -366,21 +434,21 @@ static void test_push(void) {
   ints_free(&xs);
 }
 
-static void test_pop(void) {
+static void test_pop_back(void) {
   ints_t xs = {0};
 
   for (size_t i = 0; i < 10; ++i) {
-    ints_push(&xs, (int)i);
+    ints_push_back(&xs, (int)i);
   }
   for (size_t i = 0; i < 10; ++i) {
     assert(ints_size(&xs) == 10 - i);
-    int x = ints_pop(&xs);
+    int x = ints_pop_back(&xs);
     assert(x == 10 - (int)i - 1);
   }
 
   for (size_t i = 0; i < 10; ++i) {
-    ints_push(&xs, (int)i);
-    (void)ints_pop(&xs);
+    ints_push_back(&xs, (int)i);
+    (void)ints_pop_back(&xs);
     assert(ints_is_empty(&xs));
   }
 
@@ -469,6 +537,8 @@ int main(void) {
   } while (0)
 
   RUN(create_reset);
+  RUN(init);
+  RUN(init_reset);
   RUN(append);
   RUN(get);
   RUN(set);
@@ -477,6 +547,10 @@ int main(void) {
   RUN(at);
   RUN(clear_empty);
   RUN(clear);
+  RUN(push_one);
+  RUN(push_then_pop_ten);
+  RUN(push_then_pop_many);
+  RUN(push_pop_interleaved);
   RUN(resize_empty_1);
   RUN(resize_empty_2);
   RUN(resize_down);
@@ -488,8 +562,8 @@ int main(void) {
   RUN(shrink);
   RUN(shrink_empty);
   RUN(free);
-  RUN(push);
-  RUN(pop);
+  RUN(push_back);
+  RUN(pop_back);
   RUN(large);
   RUN(attach_detach);
   RUN(dtor);
