@@ -1,16 +1,18 @@
+/// @file
+/// @brief [edge arrow shapes](https://graphviz.org/doc/info/arrows.html)
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 
 #include <assert.h>
-#include <cgraph/streq.h>
 #include <cgraph/startswith.h>
 #include <common/geomprocs.h>
 #include <common/render.h>
@@ -18,6 +20,7 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <util/streq.h>
 
 #define EPSILON .0001
 
@@ -199,7 +202,7 @@ static void arrow_match_name(char *name, uint32_t *flag) {
 	next = rest;
         rest = arrow_match_shape(next, &f);
 	if (f == ARR_TYPE_NONE) {
-	    agerr(AGWARN, "Arrow type \"%s\" unknown - ignoring\n", next);
+	    agwarningf("Arrow type \"%s\" unknown - ignoring\n", next);
 	    return;
 	}
 	if (f == ARR_TYPE_GAP && i == NUMB_OF_ARROW_HEADS - 1)
@@ -1141,10 +1144,10 @@ boxf arrow_bb(pointf p, pointf u, double arrowsize)
     dy = by + u.y;
 
     /* compute a right bb */
-    bb.UR.x = MAX(ax, MAX(bx, MAX(cx, dx)));
-    bb.UR.y = MAX(ay, MAX(by, MAX(cy, dy)));
-    bb.LL.x = MIN(ax, MIN(bx, MIN(cx, dx)));
-    bb.LL.y = MIN(ay, MIN(by, MIN(cy, dy)));
+    bb.UR.x = fmax(ax, fmax(bx, fmax(cx, dx)));
+    bb.UR.y = fmax(ay, fmax(by, fmax(cy, dy)));
+    bb.LL.x = fmin(ax, fmin(bx, fmin(cx, dx)));
+    bb.LL.y = fmin(ay, fmin(by, fmin(cy, dy)));
  
     return bb;
 }
