@@ -1,39 +1,32 @@
-/* $Id$Revision: */
-/* vim:set shiftwidth=4 ts=8: */
-
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
-
+#include <cgraph/alloc.h>
 #include <stddef.h>
-#include <intset.h>
-#include <memory.h>
+#include <common/intset.h>
 
-static void*
-mkIntItem(Dt_t* d,intitem* obj,Dtdisc_t* disc)
-{ 
-    intitem* np = NEW(intitem);
+static void *mkIntItem(void *p, Dtdisc_t *disc) {
+    (void)disc;
+    intitem *obj = p;
+
+    intitem* np = gv_alloc(sizeof(intitem));
     np->id = obj->id;
-    return (void*)np;
+    return np;
 }
 
-static void
-freeIntItem(Dt_t* d,intitem* obj,Dtdisc_t* disc)
-{
-    free (obj);
-}
-
-static int
-cmpid(Dt_t* d, int* key1, int* key2, Dtdisc_t* disc)
-{
+static int cmpid(void *k1, void *k2) {
+  const size_t *key1 = k1;
+  const size_t *key2 = k2;
   if (*key1 > *key2) return 1;
   else if (*key1 < *key2) return -1;
   else return 0;
@@ -43,12 +36,9 @@ static Dtdisc_t intSetDisc = {
     offsetof(intitem,id),
     sizeof(int),
     offsetof(intitem,link),
-    (Dtmake_f)mkIntItem,
-    (Dtfree_f)freeIntItem,
-    (Dtcompar_f)cmpid,
-    0,
-    0,
-    0
+    mkIntItem,
+    free,
+    cmpid,
 };
 
 Dt_t* 
@@ -57,18 +47,14 @@ openIntSet (void)
     return dtopen(&intSetDisc,Dtoset);
 }
 
-void 
-addIntSet (Dt_t* is, int v)
-{
+void addIntSet(Dt_t *is, size_t v) {
     intitem obj;
 
     obj.id = v;
     dtinsert(is, &obj);
 }
 
-int 
-inIntSet (Dt_t* is, int v)
-{
+int inIntSet(Dt_t *is, size_t v) {
     return (dtmatch (is, &v) != 0);
 }
 
