@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -19,15 +21,15 @@
 static int N_EPSF_files;
 static Dict_t *EPSF_contents;
 
-static void ps_image_free(usershape_t *p, Dtdisc_t *disc) {
-    (void)disc;
+static void ps_image_free(void *shape) {
+    usershape_t *p = shape;
     free(p->data);
 }
 
 static Dtdisc_t ImageDictDisc = {
     .key = offsetof(usershape_t, name),
     .size = -1,
-    .freef = (Dtfree_f)ps_image_free,
+    .freef = ps_image_free,
 };
 
 static usershape_t *user_init(const char *str)
@@ -45,7 +47,7 @@ static usershape_t *user_init(const char *str)
 	return us;
 
     if (!(fp = fopen(str, "r"))) {
-	agerr(AGWARN, "couldn't open epsf file %s\n", str);
+	agwarningf("couldn't open epsf file %s\n", str);
 	return NULL;
     }
     /* try to find size */
@@ -78,13 +80,13 @@ static usershape_t *user_init(const char *str)
             us->must_inline = must_inline;
         }
         else {
-            agerr(AGWARN, "couldn't read from epsf file %s\n", str);
+            agwarningf("couldn't read from epsf file %s\n", str);
             free(us->data);
             free(us);
             us = NULL;
         }
     } else {
-	agerr(AGWARN, "BoundingBox not found in epsf file %s\n", str);
+	agwarningf("BoundingBox not found in epsf file %s\n", str);
 	us = NULL;
     }
     fclose(fp);
@@ -100,8 +102,8 @@ void epsf_init(node_t * n)
 	usershape_t *us = user_init(str);
 	if (!us)
 	    return;
-	int dx = us->w;
-	int dy = us->h;
+	const double dx = us->w;
+	const double dy = us->h;
 	ND_width(n) = PS2INCH(dx);
 	ND_height(n) = PS2INCH(dy);
 	ND_shape_info(n) = desc = gv_alloc(sizeof(epsf_t));
@@ -109,7 +111,7 @@ void epsf_init(node_t * n)
 	desc->offset.x = -us->x - dx / 2;
 	desc->offset.y = -us->y - dy / 2;
     } else
-	agerr(AGWARN, "shapefile not set or not found for epsf node %s\n", agnameof(n));
+	agwarningf("shapefile not set or not found for epsf node %s\n", agnameof(n));
 }
 
 void epsf_free(node_t * n)
@@ -153,7 +155,7 @@ void cat_libfile(GVJ_t * job, const char **arglib, const char **stdlib)
                 continue;       /* ignore empty string */
             const char *safepath = safefile(p);    /* make sure filename is okay */
 	    if (!safepath) {
-		agerr(AGWARN, "can't find library file %s\n", p);
+		agwarningf("can't find library file %s\n", p);
 	    }
             else if ((fp = fopen(safepath, "r"))) {
                 while (true) {
@@ -167,13 +169,11 @@ void cat_libfile(GVJ_t * job, const char **arglib, const char **stdlib)
                 gvputs(job, "\n"); /* append a newline just in case */
 		fclose (fp);
             } else
-                agerr(AGWARN, "can't open library file %s\n", safepath);
+                agwarningf("can't open library file %s\n", safepath);
         }
     }
 }
 
-#define FILTER_EPSF 1
-#ifdef FILTER_EPSF
 /* this removes EPSF DSC comments that, when nested in another
  * document, cause errors in Ghostview and other Postscript
  * processors (although legal according to the Adobe EPSF spec).
@@ -203,12 +203,6 @@ void epsf_emit_body(GVJ_t *job, usershape_t *us)
 	gvputc(job, '\n');
     }
 }
-#else
-void epsf_emit_body(GVJ_t *job, usershape_t *us)
-{
-	gvputs(job, us->data);
-}
-#endif
 
 void epsf_define(GVJ_t *job)
 {
@@ -279,7 +273,7 @@ char *ps_string(char *ins, int chset)
 	    break;
 	case NONLATIN :
 	    if (!warned) {
-		agerr (AGWARN, "UTF-8 input uses non-Latin1 characters which cannot be handled by this PostScript driver\n");
+		agwarningf("UTF-8 input uses non-Latin1 characters which cannot be handled by this PostScript driver\n");
 		warned = 1;
 	    }
 	    base = ins;
