@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,7 +17,8 @@
  * component.
  */
 
-#include <cgraph/stack.h>
+#include <cgraph/alloc.h>
+#include <cgraph/list.h>
 #include <dotgen/dot.h>
 #include <stddef.h>
 #include <stdint.h>
@@ -34,7 +35,6 @@ begin_component(graph_t* g)
 static void 
 add_to_component(graph_t* g, node_t * n)
 {
-    GD_n_nodes(g)++;
     ND_mark(n) = Cmark;
     if (Last_node) {
 	ND_prev(n) = Last_node;
@@ -51,22 +51,25 @@ static void
 end_component(graph_t* g)
 {
     size_t i = GD_comp(g).size++;
-    GD_comp(g).list = ALLOC(GD_comp(g).size, GD_comp(g).list, node_t *);
+    GD_comp(g).list = gv_recalloc(GD_comp(g).list, GD_comp(g).size - 1,
+                                  GD_comp(g).size, sizeof(node_t *));
     GD_comp(g).list[i] = GD_nlist(g);
 }
 
-static void push(gv_stack_t *sp, node_t *np) {
+DEFINE_LIST(node_stack, node_t *)
+
+static void push(node_stack_t *sp, node_t *np) {
   ND_mark(np) = Cmark + 1;
-  stack_push(sp, np);
+  node_stack_push_back(sp, np);
 }
 
-static node_t *pop(gv_stack_t *sp) {
+static node_t *pop(node_stack_t *sp) {
 
-  if (stack_is_empty(sp)) {
+  if (node_stack_is_empty(sp)) {
     return NULL;
   }
 
-  return stack_pop(sp);
+  return node_stack_pop_back(sp);
 }
 
 /* search_component:
@@ -77,7 +80,7 @@ static node_t *pop(gv_stack_t *sp) {
  * in this call to decompose will have mark < Cmark; processed nodes will have mark=Cmark;
  * so we use mark = Cmark+1 to indicate nodes on the stack.
  */
-static void search_component(gv_stack_t *stk, graph_t *g, node_t *n) {
+static void search_component(node_stack_t *stk, graph_t *g, node_t *n) {
     int c;
     elist vec[4];
     node_t *other;
@@ -112,11 +115,10 @@ void decompose(graph_t * g, int pass)
 {
     graph_t *subg;
     node_t *n, *v;
-    gv_stack_t stk = {0};
+    node_stack_t stk = {0};
 
     if (++Cmark == 0)
 	Cmark = 1;
-    GD_n_nodes(g) = 0;
     GD_comp(g).size = 0;
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	v = n;
@@ -130,5 +132,5 @@ void decompose(graph_t * g, int pass)
 	    end_component(g);
 	}
     }
-    stack_reset(&stk);
+    node_stack_free(&stk);
 }
