@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,39 +17,52 @@
  */
 
 #include <assert.h>
+#include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/exit.h>
+#include <cgraph/gv_math.h>
+#include <cgraph/list.h>
+#include <cgraph/queue.h>
 #include <dotgen/dot.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+#include <util/streq.h>
+
+struct adjmatrix_t {
+  size_t nrows;
+  size_t ncols;
+  char *data;
+};
 
 /* #define DEBUG */
 #define MARK(v)		(ND_mark(v))
 #define saveorder(v)	(ND_coord(v)).x
-#define flatindex(v)	ND_low(v)
+#define flatindex(v)	((size_t)ND_low(v))
 
 	/* forward declarations */
 static bool medians(graph_t * g, int r0, int r1);
-static int nodeposcmpf(node_t ** n0, node_t ** n1);
-static int edgeidcmpf(edge_t ** e0, edge_t ** e1);
+static int nodeposcmpf(const void *, const void *);
+static int edgeidcmpf(const void *, const void *);
 static void flat_breakcycles(graph_t * g);
 static void flat_reorder(graph_t * g);
 static void flat_search(graph_t * g, node_t * v);
 static void init_mincross(graph_t * g);
 static void merge2(graph_t * g);
-static void init_mccomp(graph_t * g, int c);
+static void init_mccomp(graph_t *g, size_t c);
 static void cleanup2(graph_t * g, int nc);
-static int mincross_clust(graph_t * g, int);
-static int mincross(graph_t * g, int startpass, int endpass, int);
+static int mincross_clust(graph_t *g, ints_t *scratch);
+static int mincross(graph_t *g, int startpass, ints_t *scratch);
 static void mincross_step(graph_t * g, int pass);
 static void mincross_options(graph_t * g);
 static void save_best(graph_t * g);
 static void restore_best(graph_t * g);
-static adjmatrix_t *new_matrix(int i, int j);
+static adjmatrix_t *new_matrix(size_t i, size_t j);
 static void free_matrix(adjmatrix_t * p);
-static int ordercmpf(int *i0, int *i1);
+static int ordercmpf(const void *, const void *);
+static int ncross(ints_t *scratch);
 #ifdef DEBUG
 #if DEBUG > 1
 static int gd_minrank(Agraph_t *g) {return GD_minrank(g);}
@@ -66,7 +79,7 @@ void node_in_root_vlist(node_t * n);
 
 	/* mincross parameters */
 static int MinQuit;
-static double Convergence;
+static const double Convergence = .995;
 
 static graph_t *Root;
 static int GlobalMinRank, GlobalMaxRank;
@@ -237,8 +250,6 @@ fixLabelOrder (graph_t* g, rank_t* rk)
 {
     int cnt;
     bool haveBackedge = false;
-    Agnode_t** arr;
-    int* indices;
     Agraph_t* sg;
     Agnode_t* n;
     Agnode_t* nxtp;
@@ -259,8 +270,8 @@ fixLabelOrder (graph_t* g, rank_t* rk)
     if (!haveBackedge) return;
     
     sg = agsubg(g, "comp", 1);
-    arr = N_NEW(agnnodes(g), Agnode_t*);
-    indices = N_NEW(agnnodes(g), int);
+    Agnode_t **arr = gv_calloc(agnnodes(g), sizeof(Agnode_t*));
+    int *indices = gv_calloc(agnnodes(g), sizeof(int));
 
     for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 	if (ND_x(n) || agdegree(g,n,1,1) == 0) continue;
@@ -268,7 +279,7 @@ fixLabelOrder (graph_t* g, rank_t* rk)
 	    int i, sz = agnnodes(sg);
 	    cnt = topsort (g, sg, arr);
 	    assert (cnt == sz);
-	    qsort(indices, cnt, sizeof(int), (qsort_cmpf)ordercmpf);
+	    qsort(indices, cnt, sizeof(int), ordercmpf);
 	    for (i = 0; i < sz; i++) {
 		ND_order(arr[i]) = indices[i];
 		rk->v[indices[i]] = arr[i];
@@ -276,6 +287,7 @@ fixLabelOrder (graph_t* g, rank_t* rk)
        }
        emptyComp(sg);
     }
+    free(indices);
     free (arr);
 }
 
@@ -293,7 +305,7 @@ checkLabelOrder (graph_t* g)
 {
     int j, r, lo, hi;
     graph_t* lg = NULL;
-    char buf[BUFSIZ];
+    agxbuf buf = {0};
     rank_t* rk;
     Agnode_t* u;
     Agnode_t* n;
@@ -303,10 +315,10 @@ checkLabelOrder (graph_t* g)
 	rk = GD_rank(g)+r;
 	for (j = 0; j < rk->n; j++) {
 	    u = rk->v[j];
-	    if ((e = (edge_t*)ND_alg(u))) {
+	    if ((e = ND_alg(u))) {
 		if (!lg) lg = agopen ("lg", Agstrictdirected, 0);
-		snprintf(buf, sizeof(buf), "%d", j);
-		n = agnode(lg, buf, 1);
+		agxbprint(&buf, "%d", j);
+		n = agnode(lg, agxbuse(&buf), 1);
 		agbindrec(n, "info", sizeof(info_t), true);
 		lo = ND_order(aghead(ND_out(u).list[0]));
 		hi = ND_order(aghead(ND_out(u).list[1]));
@@ -327,6 +339,7 @@ checkLabelOrder (graph_t* g)
 	    lg = NULL;
 	}
     }
+    agxbfree(&buf);
 }
 
 /* dot_mincross:
@@ -334,9 +347,8 @@ checkLabelOrder (graph_t* g)
  * Note that nodes are not placed into GD_rank(g) until mincross()
  * is called.
  */
-void dot_mincross(graph_t * g, int doBalance)
-{
-    int c, nc;
+void dot_mincross(graph_t *g) {
+    int nc;
     char *s;
 
     /* check whether malformed input has led to empty cluster that the crossing
@@ -358,16 +370,19 @@ void dot_mincross(graph_t * g, int doBalance)
 
     init_mincross(g);
 
-    for (nc = c = 0; c < GD_comp(g).size; c++) {
-	init_mccomp(g, c);
-	nc += mincross(g, 0, 2, doBalance);
+    ints_t scratch = {0};
+
+    size_t comp;
+    for (nc = 0, comp = 0; comp < GD_comp(g).size; comp++) {
+	init_mccomp(g, comp);
+	nc += mincross(g, 0, &scratch);
     }
 
     merge2(g);
 
     /* run mincross on contents of each cluster */
-    for (c = 1; c <= GD_n_cluster(g); c++) {
-	nc += mincross_clust(GD_clust(g)[c], doBalance);
+    for (int c = 1; c <= GD_n_cluster(g); c++) {
+	nc += mincross_clust(GD_clust(g)[c], &scratch);
 #ifdef DEBUG
 	check_vlists(GD_clust(g)[c]);
 	check_order();
@@ -377,21 +392,21 @@ void dot_mincross(graph_t * g, int doBalance)
     if (GD_n_cluster(g) > 0 && (!(s = agget(g, "remincross")) || mapbool(s))) {
 	mark_lowclusters(g);
 	ReMincross = true;
-	nc = mincross(g, 2, 2, doBalance);
+	nc = mincross(g, 2, &scratch);
 #ifdef DEBUG
-	for (c = 1; c <= GD_n_cluster(g); c++)
+	for (int c = 1; c <= GD_n_cluster(g); c++)
 	    check_vlists(GD_clust(g)[c]);
 #endif
     }
+    ints_free(&scratch);
     cleanup2(g, nc);
 }
 
-static adjmatrix_t *new_matrix(int i, int j)
-{
-    adjmatrix_t *rv = NEW(adjmatrix_t);
+static adjmatrix_t *new_matrix(size_t i, size_t j) {
+    adjmatrix_t *rv = gv_alloc(sizeof(adjmatrix_t));
     rv->nrows = i;
     rv->ncols = j;
-    rv->data = N_NEW(i * j, char);
+    rv->data = gv_calloc(i * j, sizeof(char));
     return rv;
 }
 
@@ -405,8 +420,7 @@ static void free_matrix(adjmatrix_t * p)
 
 #define ELT(M,i,j)		(M->data[((i)*M->ncols)+(j)])
 
-static void init_mccomp(graph_t * g, int c)
-{
+static void init_mccomp(graph_t *g, size_t c) {
     int r;
 
     GD_nlist(g) = GD_comp(g).list[c];
@@ -425,8 +439,7 @@ static int betweenclust(edge_t * e)
     return (ND_clust(agtail(e)) != ND_clust(aghead(e)));
 }
 
-static void do_ordering_node (graph_t * g, node_t* n, int outflag)
-{
+static void do_ordering_node(graph_t *g, node_t *n, bool outflag) {
     int i, ne;
     node_t *u, *v;
     edge_t *e, *f, *fe;
@@ -448,7 +461,7 @@ static void do_ordering_node (graph_t * g, node_t* n, int outflag)
     /* write null terminator at end of list.
        requires +1 in TE_list alloccation */
     sortlist[ne] = 0;
-    qsort(sortlist, ne, sizeof(sortlist[0]), (qsort_cmpf) edgeidcmpf);
+    qsort(sortlist, ne, sizeof(sortlist[0]), edgeidcmpf);
     for (ne = 1; (f = sortlist[ne]); ne++) {
 	e = sortlist[ne - 1];
 	if (outflag) {
@@ -466,8 +479,7 @@ static void do_ordering_node (graph_t * g, node_t* n, int outflag)
     }
 }
 
-static void do_ordering(graph_t * g, int outflag)
-{
+static void do_ordering(graph_t *g, bool outflag) {
     /* Order all nodes in graph */
     node_t *n;
 
@@ -485,11 +497,11 @@ static void do_ordering_for_nodes(graph_t * g)
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if ((ordering = late_string(n, N_ordering, NULL))) {
 	    if (streq(ordering, "out"))
-		do_ordering_node(g, n, TRUE);
+		do_ordering_node(g, n, true);
 	    else if (streq(ordering, "in"))
-		do_ordering_node(g, n, FALSE);
+		do_ordering_node(g, n, false);
 	    else if (ordering[0])
-		agerr(AGERR, "ordering '%s' not recognized for node '%s'.\n", ordering, agnameof(n));
+		agerrorf("ordering '%s' not recognized for node '%s'.\n", ordering, agnameof(n));
 	}
     }
 }
@@ -509,11 +521,11 @@ static void ordered_edges(graph_t * g)
 	return;
     if ((ordering = late_string(g, G_ordering, NULL))) {
 	if (streq(ordering, "out"))
-	    do_ordering(g, TRUE);
+	    do_ordering(g, true);
 	else if (streq(ordering, "in"))
-	    do_ordering(g, FALSE);
+	    do_ordering(g, false);
 	else if (ordering[0])
-	    agerr(AGERR, "ordering '%s' not recognized.\n", ordering);
+	    agerrorf("ordering '%s' not recognized.\n", ordering);
     }
     else
     {
@@ -528,54 +540,48 @@ static void ordered_edges(graph_t * g)
     }
 }
 
-static int mincross_clust(graph_t * g, int doBalance)
-{
+static int mincross_clust(graph_t *g, ints_t *scratch) {
     int c, nc;
 
     expand_cluster(g);
     ordered_edges(g);
     flat_breakcycles(g);
     flat_reorder(g);
-    nc = mincross(g, 2, 2, doBalance);
+    nc = mincross(g, 2, scratch);
 
     for (c = 1; c <= GD_n_cluster(g); c++)
-	nc += mincross_clust(GD_clust(g)[c], doBalance);
+	nc += mincross_clust(GD_clust(g)[c], scratch);
 
     save_vlist(g);
     return nc;
 }
 
-static int left2right(graph_t * g, node_t * v, node_t * w)
-{
+static bool left2right(graph_t *g, node_t *v, node_t *w) {
     adjmatrix_t *M;
-    int rv;
 
     /* CLUSTER indicates orig nodes of clusters, and vnodes of skeletons */
     if (!ReMincross) {
 	if (ND_clust(v) != ND_clust(w) && ND_clust(v) && ND_clust(w)) {
 	    /* the following allows cluster skeletons to be swapped */
 	    if (ND_ranktype(v) == CLUSTER && ND_node_type(v) == VIRTUAL)
-		return FALSE;
+		return false;
 	    if (ND_ranktype(w) == CLUSTER && ND_node_type(w) == VIRTUAL)
-		return FALSE;
-	    return TRUE;
+		return false;
+	    return true;
 	}
     } else {
 	if (ND_clust(v) != ND_clust(w))
-	    return TRUE;
+	    return true;
     }
     M = GD_rank(g)[ND_rank(v)].flat;
     if (M == NULL)
-	rv = FALSE;
-    else {
-	if (GD_flip(g)) {
-	    node_t *t = v;
-	    v = w;
-	    w = t;
-	}
-	rv = ELT(M, flatindex(v), flatindex(w));
+	return false;
+    if (GD_flip(g)) {
+	node_t *t = v;
+	v = w;
+	w = t;
     }
-    return rv;
+    return ELT(M, flatindex(v), flatindex(w)) != 0;
 }
 
 static int in_cross(node_t * v, node_t * w)
@@ -629,131 +635,6 @@ static void exchange(node_t * v, node_t * w)
     GD_rank(Root)[r].v[vi] = w;
 }
 
-static void balanceNodes(graph_t * g, int r, node_t * v, node_t * w)
-{
-    node_t *s;			/* separator node */
-    int sepIndex = 0;
-    int nullType;		/* type of null nodes */
-    int cntDummy = 0, cntOri = 0;
-    int k = 0, m = 0, k1 = 0, m1 = 0, i = 0;
-
-    /* we only consider v and w of different types */
-    if (ND_node_type(v) == ND_node_type(w))
-	return;
-
-    /* count the number of dummy and original nodes */
-    for (i = 0; i < GD_rank(g)[r].n; i++) {
-	if (ND_node_type(GD_rank(g)[r].v[i]) == NORMAL)
-	    cntOri++;
-	else
-	    cntDummy++;
-    }
-
-    if (cntOri < cntDummy) {
-	if (ND_node_type(v) == NORMAL)
-	    s = v;
-	else
-	    s = w;
-    } else {
-	if (ND_node_type(v) == NORMAL)
-	    s = w;
-	else
-	    s = v;
-    }
-
-    /* get the separator node index */
-    for (i = 0; i < GD_rank(g)[r].n; i++) {
-	if (GD_rank(g)[r].v[i] == s)
-	    sepIndex = i;
-    }
-
-    nullType = ND_node_type(s) == NORMAL ? VIRTUAL : NORMAL;
-
-    /* count the number of null nodes to the left and 
-     * right of the separator node 
-     */
-    for (i = sepIndex - 1; i >= 0; i--) {
-	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
-	    k++;
-	else
-	    break;
-    }
-
-    for (i = sepIndex + 1; i < GD_rank(g)[r].n; i++) {
-	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
-	    m++;
-	else
-	    break;
-    }
-
-    /* now exchange v,w and calculate the same counts */
-
-    exchange(v, w);
-
-    /* get the separator node index */
-    for (i = 0; i < GD_rank(g)[r].n; i++) {
-	if (GD_rank(g)[r].v[i] == s)
-	    sepIndex = i;
-    }
-
-    /* count the number of null nodes to the left and 
-     * right of the separator node 
-     */
-    for (i = sepIndex - 1; i >= 0; i--) {
-	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
-	    k1++;
-	else
-	    break;
-    }
-
-    for (i = sepIndex + 1; i < GD_rank(g)[r].n; i++) {
-	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
-	    m1++;
-	else
-	    break;
-    }
-
-    if (abs(k1 - m1) > abs(k - m)) {
-	exchange(v, w);		//revert to the original ordering
-    }
-}
-
-static int balance(graph_t * g)
-{
-    int i, c0, c1, rv;
-    node_t *v, *w;
-    int r;
-
-    rv = 0;
-
-    for (r = GD_maxrank(g); r >= GD_minrank(g); r--) {
-
-	GD_rank(g)[r].candidate = false;
-	for (i = 0; i < GD_rank(g)[r].n - 1; i++) {
-	    v = GD_rank(g)[r].v[i];
-	    w = GD_rank(g)[r].v[i + 1];
-	    assert(ND_order(v) < ND_order(w));
-	    if (left2right(g, v, w))
-		continue;
-	    c0 = c1 = 0;
-	    if (r > 0) {
-		c0 += in_cross(v, w);
-		c1 += in_cross(w, v);
-	    }
-
-	    if (GD_rank(g)[r + 1].n > 0) {
-		c0 += out_cross(v, w);
-		c1 += out_cross(w, v);
-	    }
-
-	    if (c1 <= c0) {
-		balanceNodes(g, r, v, w);
-	    }
-	}
-    }
-    return rv;
-}
-
 static int transpose_step(graph_t * g, int r, bool reverse)
 {
     int i, c0, c1, rv;
@@ -811,13 +692,13 @@ static void transpose(graph_t * g, bool reverse)
     } while (delta >= 1);
 }
 
-static int mincross(graph_t * g, int startpass, int endpass, int doBalance)
-{
-    int maxthispass, iter, trying, pass;
+static int mincross(graph_t *g, int startpass, ints_t *scratch) {
+    const int endpass = 2;
+    int maxthispass = 0, iter, trying, pass;
     int cur_cross, best_cross;
 
     if (startpass > 1) {
-	cur_cross = best_cross = ncross(g);
+	cur_cross = best_cross = ncross(scratch);
 	save_best(g);
     } else
 	cur_cross = best_cross = INT_MAX;
@@ -825,12 +706,12 @@ static int mincross(graph_t * g, int startpass, int endpass, int doBalance)
 	if (pass <= 1) {
 	    maxthispass = MIN(4, MaxIter);
 	    if (g == dot_root(g))
-		build_ranks(g, pass);
+		build_ranks(g, pass, scratch);
 	    if (pass == 0)
 		flat_breakcycles(g);
 	    flat_reorder(g);
 
-	    if ((cur_cross = ncross(g)) <= best_cross) {
+	    if ((cur_cross = ncross(scratch)) <= best_cross) {
 		save_best(g);
 		best_cross = cur_cross;
 	    }
@@ -851,7 +732,7 @@ static int mincross(graph_t * g, int startpass, int endpass, int doBalance)
 	    if (cur_cross == 0)
 		break;
 	    mincross_step(g, iter);
-	    if ((cur_cross = ncross(g)) <= best_cross) {
+	    if ((cur_cross = ncross(scratch)) <= best_cross) {
 		save_best(g);
 		if (cur_cross < Convergence * best_cross)
 		    trying = 0;
@@ -864,12 +745,8 @@ static int mincross(graph_t * g, int startpass, int endpass, int doBalance)
     if (cur_cross > best_cross)
 	restore_best(g);
     if (best_cross > 0) {
-	transpose(g, FALSE);
-	best_cross = ncross(g);
-    }
-    if (doBalance) {
-	for (iter = 0; iter < maxthispass; iter++)
-	    balance(g);
+	transpose(g, false);
+	best_cross = ncross(scratch);
     }
 
     return best_cross;
@@ -889,7 +766,7 @@ static void restore_best(graph_t * g)
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	GD_rank(Root)[r].valid = false;
 	qsort(GD_rank(g)[r].v, GD_rank(g)[r].n, sizeof(GD_rank(g)[0].v[0]),
-	      (qsort_cmpf) nodeposcmpf);
+	      nodeposcmpf);
     }
 }
 
@@ -908,13 +785,12 @@ static void save_best(graph_t * g)
 /* merges the connected components of g */
 static void merge_components(graph_t * g)
 {
-    int c;
     node_t *u, *v;
 
     if (GD_comp(g).size <= 1)
 	return;
     u = NULL;
-    for (c = 0; c < GD_comp(g).size; c++) {
+    for (size_t c = 0; c < GD_comp(g).size; c++) {
 	v = GD_comp(g).list[c];
 	if (u)
 	    ND_next(u) = v;
@@ -1013,27 +889,24 @@ assert((rv == 0) || (ND_order(rv)-ND_order(v))*dir > 0);
     return rv;
 }
 
-static int is_a_normal_node_of(graph_t * g, node_t * v)
-{
+static bool is_a_normal_node_of(graph_t *g, node_t *v) {
     return ND_node_type(v) == NORMAL && agcontains(g, v);
 }
 
-static int is_a_vnode_of_an_edge_of(graph_t * g, node_t * v)
-{
+static bool is_a_vnode_of_an_edge_of(graph_t *g, node_t *v) {
     if (ND_node_type(v) == VIRTUAL
 	&& ND_in(v).size == 1 && ND_out(v).size == 1) {
 	edge_t *e = ND_out(v).list[0];
 	while (ED_edge_type(e) != NORMAL)
 	    e = ED_to_orig(e);
 	if (agcontains(g, e))
-	    return TRUE;
+	    return true;
     }
-    return FALSE;
+    return false;
 }
 
-static int inside_cluster(graph_t * g, node_t * v)
-{
-    return is_a_normal_node_of(g, v) | is_a_vnode_of_an_edge_of(g, v);
+static bool inside_cluster(graph_t *g, node_t *v) {
+  return is_a_normal_node_of(g, v) || is_a_vnode_of_an_edge_of(g, v);
 }
 
 static node_t *furthestnode(graph_t * g, node_t * v, int dir)
@@ -1151,7 +1024,7 @@ static void
 fillRanks (Agraph_t* g)
 {
     int rnks_sz = GD_maxrank(g) + 2;
-    int* rnks = N_NEW(rnks_sz, int);
+    int *rnks = gv_calloc(rnks_sz, sizeof(int));
     realFillRanks (g, rnks, rnks_sz, NULL);
     free (rnks);
 }
@@ -1167,8 +1040,8 @@ static void init_mincross(graph_t * g)
     Root = g;
     /* alloc +1 for the null terminator usage in do_ordering() */
     size = agnedges(dot_root(g)) + 1;
-    TE_list = N_NEW(size, edge_t *);
-    TI_list = N_NEW(size, int);
+    TE_list = gv_calloc(size, sizeof(edge_t*));
+    TI_list = gv_calloc(size, sizeof(int));
     mincross_options(g);
     if (GD_flags(g) & NEW_RANK)
 	fillRanks (g);
@@ -1214,7 +1087,7 @@ static void flat_search(graph_t * g, node_t * v)
     edge_t *e;
     adjmatrix_t *M = GD_rank(g)[ND_rank(v)].flat;
 
-    ND_mark(v) = TRUE;
+    ND_mark(v) = true;
     ND_onstack(v) = true;
     hascl = GD_n_cluster(dot_root(g)) > 0;
     if (ND_flat_out(v).list)
@@ -1252,12 +1125,12 @@ static void flat_breakcycles(graph_t * g)
 	flat = 0;
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
 	    v = GD_rank(g)[r].v[i];
-	    ND_mark(v) = FALSE;
+	    ND_mark(v) = false;
 	    ND_onstack(v) = false;
-	    flatindex(v) = i;
+	    ND_low(v) = i;
 	    if (ND_flat_out(v).size > 0 && flat == 0) {
 		GD_rank(g)[r].flat =
-		    new_matrix(GD_rank(g)[r].n, GD_rank(g)[r].n);
+		    new_matrix((size_t)GD_rank(g)[r].n, (size_t)GD_rank(g)[r].n);
 		flat = 1;
 	    }
 	}
@@ -1277,11 +1150,11 @@ static void flat_breakcycles(graph_t * g)
  */
 void allocate_ranks(graph_t * g)
 {
-    int r, low, high, *cn;
+    int r, low, high;
     node_t *n;
     edge_t *e;
 
-    cn = N_NEW(GD_maxrank(g) + 2, int);	/* must be 0 based, not GD_minrank */
+    int *cn = gv_calloc(GD_maxrank(g) + 2, sizeof(int)); // must be 0 based, not GD_minrank
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	cn[ND_rank(n)]++;
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
@@ -1296,10 +1169,10 @@ void allocate_ranks(graph_t * g)
 		cn[r]++;
 	}
     }
-    GD_rank(g) = N_NEW(GD_maxrank(g) + 2, rank_t);
+    GD_rank(g) = gv_calloc(GD_maxrank(g) + 2, sizeof(rank_t));
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	GD_rank(g)[r].an = GD_rank(g)[r].n = cn[r];
-	GD_rank(g)[r].av = GD_rank(g)[r].v = N_NEW(cn[r] + 1, node_t *);
+	GD_rank(g)[r].an = GD_rank(g)[r].n = cn[r] + 1;
+	GD_rank(g)[r].av = GD_rank(g)[r].v = gv_calloc(cn[r] + 1, sizeof(node_t*));
     }
     free(cn);
 }
@@ -1312,7 +1185,7 @@ void install_in_rank(graph_t * g, node_t * n)
     r = ND_rank(n);
     i = GD_rank(g)[r].n;
     if (GD_rank(g)[r].an <= 0) {
-	agerr(AGERR, "install_in_rank, line %d: %s %s rank %d i = %d an = 0\n",
+	agerrorf("install_in_rank, line %d: %s %s rank %d i = %d an = 0\n",
 	      __LINE__, agnameof(g), agnameof(n), r, i);
 	return;
     }
@@ -1332,18 +1205,18 @@ void install_in_rank(graph_t * g, node_t * n)
     }
 #endif
     if (ND_order(n) > GD_rank(Root)[r].an) {
-	agerr(AGERR, "install_in_rank, line %d: ND_order(%s) [%d] > GD_rank(Root)[%d].an [%d]\n",
+	agerrorf("install_in_rank, line %d: ND_order(%s) [%d] > GD_rank(Root)[%d].an [%d]\n",
 	      __LINE__, agnameof(n), ND_order(n), r, GD_rank(Root)[r].an);
 	return;
     }
     if (r < GD_minrank(g) || r > GD_maxrank(g)) {
-	agerr(AGERR, "install_in_rank, line %d: rank %d not in rank range [%d,%d]\n",
+	agerrorf("install_in_rank, line %d: rank %d not in rank range [%d,%d]\n",
 	      __LINE__, r, GD_minrank(g), GD_maxrank(g));
 	return;
     }
     if (GD_rank(g)[r].v + ND_order(n) >
 	GD_rank(g)[r].av + GD_rank(Root)[r].an) {
-	agerr(AGERR, "install_in_rank, line %d: GD_rank(g)[%d].v + ND_order(%s) [%d] > GD_rank(g)[%d].av + GD_rank(Root)[%d].an [%d]\n",
+	agerrorf("install_in_rank, line %d: GD_rank(g)[%d].v + ND_order(%s) [%d] > GD_rank(g)[%d].av + GD_rank(Root)[%d].an [%d]\n",
 	      __LINE__, r, agnameof(n),ND_order(n), r, r, GD_rank(Root)[r].an);
 	return;
     }
@@ -1353,16 +1226,13 @@ void install_in_rank(graph_t * g, node_t * n)
  *	graphs such as trees are drawn with no crossings.  it tries searching
  *	in- and out-edges and takes the better of the two initial orderings.
  */
-void build_ranks(graph_t * g, int pass)
-{
+void build_ranks(graph_t *g, int pass, ints_t *scratch) {
     int i, j;
-    node_t *n, *n0;
+    node_t *n, *n0, *ns;
     edge_t **otheredges;
-    nodequeue *q;
-
-    q = new_queue(GD_n_nodes(g));
+    queue_t q = {0};
     for (n = GD_nlist(g); n; n = ND_next(n))
-	MARK(n) = FALSE;
+	MARK(n) = false;
 
 #ifdef DEBUG
     {
@@ -1379,101 +1249,108 @@ void build_ranks(graph_t * g, int pass)
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++)
 	GD_rank(g)[i].n = 0;
 
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
+    const bool walkbackwards = g != agroot(g); // if this is a cluster, need to
+                                               // walk GD_nlist backward to
+                                               // preserve input node order
+    if (walkbackwards) {
+	for (ns = GD_nlist(g); ND_next(ns); ns = ND_next(ns)) {
+	    ;
+	}
+    } else {
+	ns = GD_nlist(g);
+    }
+    for (n = ns; n; n = walkbackwards ? ND_prev(n) : ND_next(n)) {
 	otheredges = pass == 0 ? ND_in(n).list : ND_out(n).list;
 	if (otheredges[0] != NULL)
 	    continue;
 	if (!MARK(n)) {
-	    MARK(n) = TRUE;
-	    enqueue(q, n);
-	    while ((n0 = dequeue(q))) {
+	    MARK(n) = true;
+	    queue_push(&q, n);
+	    while ((n0 = queue_pop(&q))) {
 		if (ND_ranktype(n0) != CLUSTER) {
 		    install_in_rank(g, n0);
-		    enqueue_neighbors(q, n0, pass);
+		    enqueue_neighbors(&q, n0, pass);
 		} else {
-		    install_cluster(g, n0, pass, q);
+		    install_cluster(g, n0, pass, &q);
 		}
 	    }
 	}
     }
-    if (dequeue(q))
-	agerr(AGERR, "surprise\n");
+    assert(queue_pop(&q) == NULL);
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
 	GD_rank(Root)[i].valid = false;
 	if (GD_flip(g) && GD_rank(g)[i].n > 0) {
-	    int n, ndiv2;
 	    node_t **vlist = GD_rank(g)[i].v;
-	    n = GD_rank(g)[i].n - 1;
-	    ndiv2 = n / 2;
-	    for (j = 0; j <= ndiv2; j++)
-		exchange(vlist[j], vlist[n - j]);
+	    int num_nodes_1 = GD_rank(g)[i].n - 1;
+	    int half_num_nodes_1 = num_nodes_1 / 2;
+	    for (j = 0; j <= half_num_nodes_1; j++)
+		exchange(vlist[j], vlist[num_nodes_1 - j]);
 	}
     }
 
-    if (g == dot_root(g) && ncross(g) > 0)
-	transpose(g, FALSE);
-    free_queue(q);
+    if (g == dot_root(g) && ncross(scratch) > 0)
+	transpose(g, false);
+    queue_free(&q);
 }
 
-void enqueue_neighbors(nodequeue * q, node_t * n0, int pass)
-{
-    int i;
+void enqueue_neighbors(queue_t *q, node_t *n0, int pass) {
     edge_t *e;
 
     if (pass == 0) {
-	for (i = 0; i < ND_out(n0).size; i++) {
+	for (size_t i = 0; i < ND_out(n0).size; i++) {
 	    e = ND_out(n0).list[i];
 	    if (!MARK(aghead(e))) {
-		MARK(aghead(e)) = TRUE;
-		enqueue(q, aghead(e));
+		MARK(aghead(e)) = true;
+		queue_push(q, aghead(e));
 	    }
 	}
     } else {
-	for (i = 0; i < ND_in(n0).size; i++) {
+	for (size_t i = 0; i < ND_in(n0).size; i++) {
 	    e = ND_in(n0).list[i];
 	    if (!MARK(agtail(e))) {
-		MARK(agtail(e)) = TRUE;
-		enqueue(q, agtail(e));
+		MARK(agtail(e)) = true;
+		queue_push(q, agtail(e));
 	    }
 	}
     }
 }
 
-static int constraining_flat_edge(Agraph_t *g, Agnode_t *v, Agedge_t *e)
-{
-	if (ED_weight(e) == 0) return FALSE;
-	if (!inside_cluster(g,agtail(e))) return FALSE;
-	if (!inside_cluster(g,aghead(e))) return FALSE;
-	return TRUE;
+static bool constraining_flat_edge(Agraph_t *g, Agedge_t *e) {
+  if (ED_weight(e) == 0)
+    return false;
+  if (!inside_cluster(g, agtail(e)))
+    return false;
+  if (!inside_cluster(g, aghead(e)))
+    return false;
+  return true;
 }
 
+DEFINE_LIST(nodes, node_t *)
 
 /* construct nodes reachable from 'here' in post-order.
 * This is the same as doing a topological sort in reverse order.
 */
-static int postorder(graph_t * g, node_t * v, node_t ** list, int r)
-{
+static void postorder(graph_t *g, node_t *v, nodes_t *list, int r) {
     edge_t *e;
-    int i, cnt = 0;
+    int i;
 
-    MARK(v) = TRUE;
+    MARK(v) = true;
     if (ND_flat_out(v).size > 0) {
 	for (i = 0; (e = ND_flat_out(v).list[i]); i++) {
-	    if (!constraining_flat_edge(g,v,e)) continue;
+	    if (!constraining_flat_edge(g, e)) continue;
 	    if (!MARK(aghead(e)))
-		cnt += postorder(g, aghead(e), list + cnt, r);
+		postorder(g, aghead(e), list, r);
 	}
     }
     assert(ND_rank(v) == r);
-    list[cnt++] = v;
-    return cnt;
+    nodes_append(list, v);
 }
 
 static void flat_reorder(graph_t * g)
 {
-    int i, j, r, pos, n_search, local_in_cnt, local_out_cnt, base_order;
-    node_t *v, **left, **right, *t;
-    node_t **temprank = NULL;
+    int i, r, local_in_cnt, local_out_cnt, base_order;
+    node_t *v;
+    nodes_t temprank = {0};
     edge_t *flat_e, *e;
 
     if (!GD_has_flat_edges(g))
@@ -1482,9 +1359,8 @@ static void flat_reorder(graph_t * g)
 	if (GD_rank(g)[r].n == 0) continue;
 	base_order = ND_order(GD_rank(g)[r].v[0]);
 	for (i = 0; i < GD_rank(g)[r].n; i++)
-	    MARK(GD_rank(g)[r].v[i]) = FALSE;
-	temprank = ALLOC(i + 1, temprank, node_t *);
-	pos = 0;
+	    MARK(GD_rank(g)[r].v[i]) = false;
+	nodes_clear(&temprank);
 
 	/* construct reverse topological sort order in temprank */
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
@@ -1492,39 +1368,29 @@ static void flat_reorder(graph_t * g)
 	    else v = GD_rank(g)[r].v[GD_rank(g)[r].n - i - 1];
 
 	    local_in_cnt = local_out_cnt = 0;
-	    for (j = 0; j < ND_flat_in(v).size; j++) {
+	    for (size_t j = 0; j < ND_flat_in(v).size; j++) {
 		flat_e = ND_flat_in(v).list[j];
-		if (constraining_flat_edge(g,v,flat_e)) local_in_cnt++;
+		if (constraining_flat_edge(g, flat_e)) local_in_cnt++;
 	    }
-	    for (j = 0; j < ND_flat_out(v).size; j++) {
+	    for (size_t j = 0; j < ND_flat_out(v).size; j++) {
 		flat_e = ND_flat_out(v).list[j];
-		if (constraining_flat_edge(g,v,flat_e)) local_out_cnt++;
+		if (constraining_flat_edge(g, flat_e)) local_out_cnt++;
 	    }
 	    if ((local_in_cnt == 0) && (local_out_cnt == 0))
-		temprank[pos++] = v;
+		nodes_append(&temprank, v);
 	    else {
 		if (!MARK(v) && local_in_cnt == 0) {
-		    left = temprank + pos;
-		    n_search = postorder(g, v, left, r);
-		    pos += n_search;
+		    postorder(g, v, &temprank, r);
 		}
 	    }
 	}
 
-	if (pos) {
+	if (nodes_size(&temprank) > 0) {
 	    if (!GD_flip(g)) {
-		left = temprank;
-		right = temprank + pos - 1;
-		while (left < right) {
-		    t = *left;
-		    *left = *right;
-		    *right = t;
-		    left++;
-		    right--;
-		}
+		nodes_reverse(&temprank);
 	    }
 	    for (i = 0; i < GD_rank(g)[r].n; i++) {
-		v = GD_rank(g)[r].v[i] = temprank[i];
+		v = GD_rank(g)[r].v[i] = nodes_get(&temprank, (size_t)i);
 		ND_order(v) = i + base_order;
 	    }
 
@@ -1532,10 +1398,10 @@ static void flat_reorder(graph_t * g)
 	    for (i = 0; i < GD_rank(g)[r].n; i++) {
 		v = GD_rank(g)[r].v[i];
 		if (ND_flat_out(v).list) {
-		    for (j = 0; (e = ND_flat_out(v).list[j]); j++) {
+		    for (size_t j = 0; (e = ND_flat_out(v).list[j]); j++) {
 			if ( (!GD_flip(g) && ND_order(aghead(e)) < ND_order(agtail(e))) ||
 				 ( (GD_flip(g)) && (ND_order(aghead(e)) > ND_order(agtail(e)) ))) {
-			    assert(!constraining_flat_edge(g,v,e));
+			    assert(!constraining_flat_edge(g, e));
 			    delete_flat_edge(e);
 			    j--;
 			    flat_rev(g, e);
@@ -1548,8 +1414,7 @@ static void flat_reorder(graph_t * g)
 	/* else do no harm! */
 	GD_rank(Root)[r].valid = false;
     }
-    if (temprank)
-	free(temprank);
+    nodes_free(&temprank);
 }
 
 static void reorder(graph_t * g, int r, bool reverse, bool hasfixed)
@@ -1584,9 +1449,9 @@ static void reorder(graph_t * g, int r, bool reverse, bool hasfixed)
 	    if (rp >= ep)
 		break;
 	    if (!muststay) {
-		int p1 = ND_mval(*lp);
-		int p2 = ND_mval(*rp);
-		if (p1 > p2 || (p1 == p2 && reverse)) {
+		const double p1 = ND_mval(*lp);
+		const double p2 = ND_mval(*rp);
+		if (p1 > p2 || (p1 >= p2 && reverse)) {
 		    exchange(*lp, *rp);
 		    changed++;
 		}
@@ -1634,13 +1499,10 @@ static void mincross_step(graph_t * g, int pass)
 
 static int local_cross(elist l, int dir)
 {
-    int i, j, is_out;
+    int i, j;
     int cross = 0;
     edge_t *e, *f;
-    if (dir > 0)
-	is_out = TRUE;
-    else
-	is_out = FALSE;
+    bool is_out = dir > 0;
     for (i = 0; (e = l.list[i]); i++) {
 	if (is_out)
 	    for (j = i + 1; (f = l.list[j]); j++) {
@@ -1657,9 +1519,7 @@ static int local_cross(elist l, int dir)
     return cross;
 }
 
-static int rcross(graph_t * g, int r)
-{
-    static int *Count, C;
+static int rcross(graph_t *g, int r, ints_t *Count) {
     int top, bot, cross, max, i, k;
     node_t **rtop, *v;
 
@@ -1667,27 +1527,28 @@ static int rcross(graph_t * g, int r)
     max = 0;
     rtop = GD_rank(g)[r].v;
 
-    if (C <= GD_rank(Root)[r + 1].n) {
-	C = GD_rank(Root)[r + 1].n + 1;
-	Count = ALLOC(C, Count, int);
-    }
-
-    for (i = 0; i < GD_rank(g)[r + 1].n; i++)
-	Count[i] = 0;
+    // discard any data from previous runs
+    ints_clear(Count);
 
     for (top = 0; top < GD_rank(g)[r].n; top++) {
 	edge_t *e;
 	if (max > 0) {
 	    for (i = 0; (e = ND_out(rtop[top]).list[i]); i++) {
 		for (k = ND_order(aghead(e)) + 1; k <= max; k++)
-		    cross += Count[k] * ED_xpenalty(e);
+		    cross += ints_size(Count) <= (size_t)k
+		             ? 0
+		             : ints_get(Count, (size_t)k) * ED_xpenalty(e);
 	    }
 	}
 	for (i = 0; (e = ND_out(rtop[top]).list[i]); i++) {
 	    int inv = ND_order(aghead(e));
 	    if (inv > max)
 		max = inv;
-	    Count[inv] += ED_xpenalty(e);
+	    const size_t inv_z = (size_t)inv;
+	    if (ints_size(Count) <= inv_z) {
+		ints_resize(Count, inv_z + 1, 0);
+	    }
+	    ints_set(Count, inv_z, ints_get(Count, inv_z) + ED_xpenalty(e));
 	}
     }
     for (top = 0; top < GD_rank(g)[r].n; top++) {
@@ -1703,17 +1564,17 @@ static int rcross(graph_t * g, int r)
     return cross;
 }
 
-int ncross(graph_t * g)
-{
+static int ncross(ints_t *scratch) {
+    assert(scratch != NULL);
     int r, count, nc;
 
-    g = Root;
+    graph_t *g = Root;
     count = 0;
     for (r = GD_minrank(g); r < GD_maxrank(g); r++) {
 	if (GD_rank(g)[r].valid)
 	    count += GD_rank(g)[r].cache_nc;
 	else {
-	    nc = GD_rank(g)[r].cache_nc = rcross(g, r);
+	    nc = GD_rank(g)[r].cache_nc = rcross(g, r, scratch);
 	    count += nc;
 	    GD_rank(g)[r].valid = true;
 	}
@@ -1721,9 +1582,16 @@ int ncross(graph_t * g)
     return count;
 }
 
-static int ordercmpf(int *i0, int *i1)
-{
-    return *i0 - *i1;
+static int ordercmpf(const void *x, const void *y) {
+  const int *i0 = x;
+  const int *i1 = y;
+  if (*i0 < *i1) {
+    return -1;
+  }
+  if (*i0 > *i1) {
+    return 1;
+  }
+  return 0;
 }
 
 /* flat_mval:
@@ -1770,7 +1638,7 @@ static bool flat_mval(node_t * n)
 
 static bool medians(graph_t * g, int r0, int r1)
 {
-    int i, j, j0, lm, rm, lspan, rspan, *list;
+    int i, j0, lspan, rspan, *list;
     node_t *n, **v;
     edge_t *e;
     bool hasfixed = false;
@@ -1779,7 +1647,7 @@ static bool medians(graph_t * g, int r0, int r1)
     v = GD_rank(g)[r0].v;
     for (i = 0; i < GD_rank(g)[r0].n; i++) {
 	n = v[i];
-	j = 0;
+	size_t j = 0;
 	if (r1 > r0)
 	    for (j0 = 0; (e = ND_out(n).list[j0]); j0++) {
 		if (ED_xpenalty(e) > 0)
@@ -1800,13 +1668,13 @@ static bool medians(graph_t * g, int r0, int r1)
 	    ND_mval(n) = (list[0] + list[1]) / 2;
 	    break;
 	default:
-	    qsort(list, j, sizeof(int), (qsort_cmpf) ordercmpf);
+	    qsort(list, j, sizeof(int), ordercmpf);
 	    if (j % 2)
 		ND_mval(n) = list[j / 2];
 	    else {
 		/* weighted median */
-		rm = j / 2;
-		lm = rm - 1;
+		size_t rm = j / 2;
+		size_t lm = rm - 1;
 		rspan = list[j - 1] - list[rm];
 		lspan = list[lm] - list[0];
 		if (lspan == rspan)
@@ -1826,14 +1694,48 @@ static bool medians(graph_t * g, int r0, int r1)
     return hasfixed;
 }
 
-static int nodeposcmpf(node_t ** n0, node_t ** n1)
-{
-    return (ND_order(*n0) - ND_order(*n1));
+static int nodeposcmpf(const void *x, const void *y) {
+// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
+// as the later usage is const. We need the cast because the macros use
+// non-const pointers for genericity.
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+  node_t **n0 = (node_t **)x;
+  node_t **n1 = (node_t **)y;
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+  if (ND_order(*n0) < ND_order(*n1)) {
+    return -1;
+  }
+  if (ND_order(*n0) > ND_order(*n1)) {
+    return 1;
+  }
+  return 0;
 }
 
-static int edgeidcmpf(edge_t ** e0, edge_t ** e1)
-{
-    return (AGSEQ(*e0) - AGSEQ(*e1));
+static int edgeidcmpf(const void *x, const void *y) {
+// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
+// as the later usage is const. We need the cast because the macros use
+// non-const pointers for genericity.
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+  edge_t **e0 = (edge_t **)x;
+  edge_t **e1 = (edge_t **)y;
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+  if (AGSEQ(*e0) < AGSEQ(*e1)) {
+    return -1;
+  }
+  if (AGSEQ(*e0) > AGSEQ(*e1)) {
+    return 1;
+  }
+  return 0;
 }
 
 /* following code deals with weights of edges of "virtual" nodes */
@@ -1870,7 +1772,7 @@ void virtual_weight(edge_t * e)
     /* check whether the upcoming computation will overflow */
     assert(t >= 0);
     if (INT_MAX / t < ED_weight(e)) {
-	agerr(AGERR, "overflow when calculating virtual weight of edge\n");
+	agerrorf("overflow when calculating virtual weight of edge\n");
 	graphviz_exit(EXIT_FAILURE);
     }
 
@@ -1928,7 +1830,6 @@ static void mincross_options(graph_t * g)
     /* set default values */
     MinQuit = 8;
     MaxIter = 24;
-    Convergence = .995;
 
     p = agget(g, "mclimit");
     if (p && (f = atof(p)) > 0.0) {
