@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -30,6 +30,7 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+#include <math.h>
 #include <stdlib.h>
 #include <time.h>
 #ifndef _WIN32
@@ -72,12 +73,8 @@ typedef struct {
     double T0;          /* initial temperature */
     int smode;          /* seed mode */
     double Cell;	/* grid cell size */
-    double Cell2;	/* Cell*Cell */
-    double K2;		/* K*K */
     double Wd;		/* half-width of boundary */
     double Ht;		/* half-height of boundary */
-    double Wd2;		/* Wd*Wd */
-    double Ht2;		/* Ht*Ht */
     int pass1;		/* iterations used in pass 1 */
     int loopcnt;        /* actual iterations in this pass */
 } parms_t;
@@ -96,12 +93,8 @@ static parms_t parms;
 #define T_T0        (parms.T0)
 #define T_smode     (parms.smode)
 #define T_Cell      (parms.Cell)
-#define T_Cell2     (parms.Cell2)
-#define T_K2        (parms.K2)
 #define T_Wd        (parms.Wd)
 #define T_Ht        (parms.Ht)
-#define T_Wd2       (parms.Wd2)
-#define T_Ht2       (parms.Ht2)
 #define T_pass1     (parms.pass1)
 #define T_loopcnt   (parms.loopcnt)
 
@@ -190,17 +183,15 @@ void fdp_initParams(graph_t * g)
     T_seed = DFLT_seed;
     T_smode = setSeed (g, DFLT_smode, &T_seed);
     if (T_smode == INIT_SELF) {
-	agerr(AGWARN, "fdp does not support start=self - ignoring\n");
+	agwarningf("fdp does not support start=self - ignoring\n");
 	T_seed = DFLT_smode;
     }
 
     T_pass1 = T_unscaled * T_maxIters / 100;
-    T_K2 = T_K * T_K;
 
     if (T_useGrid) {
 	if (T_Cell <= 0.0)
 	    T_Cell = 3 * T_K;
-	T_Cell2 = T_Cell * T_Cell;
     }
 #ifdef DEBUG
     if (Verbose) {
@@ -226,9 +217,9 @@ doRep(node_t * p, node_t * q, double xdelta, double ydelta, double dist2)
     }
     if (T_useNew) {
 	dist = sqrt(dist2);
-	force = T_K2 / (dist * dist2);
+	force = T_K * T_K / (dist * dist2);
     } else
-	force = T_K2 / dist2;
+	force = T_K * T_K / dist2;
     if (IS_PORT(p) && IS_PORT(q))
 	force *= 10.0;
     DISP(q)[0] += xdelta * force;
@@ -274,22 +265,20 @@ static void doNeighbor(Grid * grid, int i, int j, node_list * nodes)
 		xdelta = (ND_pos(q))[0] - (ND_pos(p))[0];
 		ydelta = (ND_pos(q))[1] - (ND_pos(p))[1];
 		dist2 = xdelta * xdelta + ydelta * ydelta;
-		if (dist2 < T_Cell2)
+		if (dist2 < T_Cell * T_Cell)
 		    doRep(p, q, xdelta, ydelta, dist2);
 	    }
 	}
     }
 }
 
-static int gridRepulse(Dt_t * dt, cell * cellp, Grid * grid)
-{
+static int gridRepulse(cell *cellp, Grid *grid) {
     node_list *nodes = cellp->nodes;
     int i = cellp->p.i;
     int j = cellp->p.j;
     node_list *p;
     node_list *q;
 
-    (void)dt;
 #ifdef DEBUG
     if (Verbose >= 3) {
 	prIndent();
@@ -373,7 +362,7 @@ static void updatePos(Agraph_t * g, double temp, bport_t * pp)
 
 	/* if ports, limit by boundary */
 	if (pp) {
-	    d = sqrt(x * x / T_Wd2 + y * y / T_Ht2);
+	    d = sqrt(x * x / (T_Wd * T_Wd) + y * y / (T_Ht * T_Ht));
 	    if (IS_PORT(n)) {
 		ND_pos(n)[0] = x / d;
 		ND_pos(n)[1] = y / d;
@@ -524,15 +513,13 @@ static pointf initPositions(graph_t * g, bport_t * pp)
     } else {
 	ctr.x = ctr.y = 0;
     }
-    T_Wd2 = T_Wd * T_Wd;
-    T_Ht2 = T_Ht * T_Ht;
 
     /* Set seed value */
     if (T_smode == INIT_RANDOM)
 	local_seed = T_seed;
     else {
 #if defined(_WIN32)
-	local_seed = time(NULL);
+	local_seed = (long)time(NULL);
 #else
 	local_seed = getpid() ^ time(NULL);
 #endif
@@ -549,13 +536,11 @@ static pointf initPositions(graph_t * g, bport_t * pp)
      * TODO: place unfixed points using adjacent ports or fixed pts.
      */
     if (pp) {
-/* fprintf (stderr, "initPos %s ctr (%g,%g) Wd %g Ht %g\n", agnameof(g), ctr.x, ctr.y, T_Wd, T_Ht); */
 	while (pp->e) {		/* position ports on ellipse */
 	    np = pp->n;
 	    ND_pos(np)[0] = T_Wd * cos(pp->alpha) + ctr.x;
 	    ND_pos(np)[1] = T_Ht * sin(pp->alpha) + ctr.y;
 	    ND_pinned(np) = P_SET;
-/* fprintf (stderr, "%s pt (%g,%g) %g\n", agnameof(np), ND_pos(np)[0], ND_pos(np)[1], pp->alpha); */
 	    pp++;
 	}
 	for (np = agfstnode(g); np; np = agnxtnode(g, np)) {
@@ -587,17 +572,14 @@ static pointf initPositions(graph_t * g, bport_t * pp)
 		if (cnt > 1) {
 		    ND_pos(np)[0] = p.x;
 		    ND_pos(np)[1] = p.y;
-/* fprintf (stderr, "%s 1 (%g,%g)\n", agnameof(np), p.x, p.y); */
 		} else if (cnt == 1) {
 		    ND_pos(np)[0] = 0.98 * p.x + 0.1 * ctr.x;
 		    ND_pos(np)[1] = 0.9 * p.y + 0.1 * ctr.y;
-/* fprintf (stderr, "%s %d (%g,%g)\n", agnameof(np), cnt, ND_pos(np)[0], ND_pos(np)[1]); */
 		} else {
 		    double angle = PItimes2 * drand48();
 		    double radius = 0.9 * drand48();
 		    ND_pos(np)[0] = radius * T_Wd * cos(angle);
 		    ND_pos(np)[1] = radius * T_Ht * sin(angle);
-/* fprintf (stderr, "%s 0 (%g,%g)\n", agnameof(np), ND_pos(np)[0], ND_pos(np)[1]); */
 		}
 		ND_pinned(np) = P_SET;
 	    }
@@ -624,32 +606,6 @@ static pointf initPositions(graph_t * g, bport_t * pp)
     return ctr;
 }
 
-void dumpstat(graph_t * g)
-{
-    double dx, dy;
-    double l, max2 = 0.0;
-    node_t *np;
-    edge_t *ep;
-    for (np = agfstnode(g); np; np = agnxtnode(g, np)) {
-	dx = DISP(np)[0];
-	dy = DISP(np)[1];
-	l = dx * dx + dy * dy;
-	if (l > max2)
-	    max2 = l;
-	fprintf(stderr, "%s: (%f,%f) (%f,%f)\n", agnameof(np),
-		ND_pos(np)[0], ND_pos(np)[1], DISP(np)[0], DISP(np)[1]);
-    }
-    fprintf(stderr, "max delta = %f\n", sqrt(max2));
-    for (np = agfstnode(g); np; np = agnxtnode(g, np)) {
-	for (ep = agfstout(g, np); ep; ep = agnxtout(g, ep)) {
-	    dx = ND_pos(np)[0] - ND_pos(aghead(ep))[0];
-	    dy = ND_pos(np)[1] - ND_pos(aghead(ep))[1];
-	    fprintf(stderr, "  %s --  %s  (%f)\n", agnameof(np),
-		    agnameof(aghead(ep)), sqrt(dx * dx + dy * dy));
-	}
-    }
-}
-
 /* fdp_tLayout:
  * Given graph g with ports nodes, layout g respecting ports.
  * If some node have position information, it may be useful to
@@ -691,7 +647,6 @@ void fdp_tLayout(graph_t * g, xparams * xpms)
 	    ND_pos(n)[1] += ctr.y;
 	}
     }
-/* dumpstat (g); */
     if (reset)
 	reset_params();
 }
