@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -19,13 +19,10 @@ extern "C" {
 #endif
 
 //creates a font file with given name and font description
-//returns non-zero if fails
     unsigned char *glCompCreatePangoTexture(char *fontdescription,
 					int fontsize, char *txt,
 					cairo_surface_t * surface, int *w,
 					int *h);
-    int glCompCreateFontFile(char *fontdescription, int fs, char *fontfile,
-			 float gw, float gh);
 
 #ifdef __cplusplus
 }
