@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -22,20 +22,21 @@
 #include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
+#include <cgraph/gv_ctype.h>
+#include <cgraph/ingraphs.h>
 #include <cgraph/exit.h>
-#include <cgraph/stack.h>
+#include <cgraph/list.h>
+#include <cgraph/queue.h>
+#include <cgraph/unreachable.h>
 #include <common/globals.h>
-#include <ingraphs/ingraphs.h>
 #include <gvpr/compile.h>
-#include <gvpr/queue.h>
 #include <gvpr/gvpr.h>
 #include <gvpr/actions.h>
-#include <ast/sfstr.h>
 #include <ast/error.h>
 #include <stdbool.h>
 #include <stddef.h>
+#include <stdio.h>
 #include <string.h>
-#include <ctype.h>
 #include <setjmp.h>
 #include <getopt.h>
 
@@ -64,7 +65,7 @@ If no files are specified, stdin is used\n";
 
 typedef struct {
     char *cmdName;              /* command name */
-    Sfio_t *outFile;		/* output stream; stdout default */
+    FILE *outFile;		/* output stream; stdout default */
     char *program;              /* program source */
     int useFile;		/* true if program comes from a file */
     int compflags;
@@ -76,11 +77,8 @@ typedef struct {
     int verbose;
 } options;
 
-static Sfio_t *openOut(char *name)
-{
-    Sfio_t *outs;
-
-    outs = sfopen(name, "w");
+static FILE *openOut(char *name) {
+    FILE *outs = fopen(name, "w");
     if (outs == 0) {
 	error(ERROR_ERROR, "could not open %s for writing", name);
     }
@@ -106,7 +104,7 @@ static char *gettok(char **sp)
     char c;
     char q = '\0';		/* if non-0, in quote mode with quote char q */
 
-    while (isspace((int)*rs))
+    while (gv_isspace(*rs))
 	rs++;
     if ((c = *rs) == '\0')
 	return NULL;
@@ -125,7 +123,7 @@ static char *gettok(char **sp)
 		      "backslash in argument followed by no character - ignored");
 		rs--;
 	    }
-	} else if (q || !isspace(c))
+	} else if (q || !gv_isspace(c))
 	    *ws++ = c;
 	else
 	    break;
@@ -200,11 +198,8 @@ concat (char* pfx, char* sfx)
  * If arg contains '/', return arg.
  * Else search directories in GVPRPATH for arg.
  * Return NULL on error.
- * 
- * FIX - use pathinclude/pathfind
  */
-static char *resolve(char *arg, int Verbose)
-{
+static char *resolve(char *arg, int verbose) {
     char *path;
     char *s;
     char *cp;
@@ -229,7 +224,7 @@ static char *resolve(char *arg, int Verbose)
     }
     else
 	path = DFLT_GVPRPATH;
-    if (Verbose)
+    if (verbose)
 	fprintf (stderr, "PATH: %s\n", path);
     agxbuf fp = {0};
 
@@ -260,7 +255,7 @@ static char *resolve(char *arg, int Verbose)
 
     agxbfree(&fp);
     free(pathp);
-    if (Verbose)
+    if (verbose)
 	fprintf (stderr, "file %s resolved to %s\n", arg, fname);
     return fname;
 }
@@ -359,8 +354,8 @@ doFlags(char* arg, int argi, int argc, char** argv, options* opts)
 }
 
 static void freeOpts(options opts) {
-    if (opts.outFile != sfstdout)
-	sfclose(opts.outFile);
+    if (opts.outFile != NULL && opts.outFile != stdout)
+	fclose(opts.outFile);
     free(opts.inFiles);
     if (opts.useFile)
 	free(opts.program);
@@ -373,7 +368,6 @@ static void freeOpts(options opts) {
  * Parse command line options.
  */
 static options scanArgs(int argc, char **argv) {
-    int i, nfiles;
     char* arg;
     options opts = {0};
 
@@ -384,15 +378,15 @@ static options scanArgs(int argc, char **argv) {
     opts.verbose = 0;
 
     /* estimate number of file names */
-    nfiles = 0;
-    for (i = 1; i < argc; i++)
+    size_t nfiles = 0;
+    for (int i = 1; i < argc; i++)
 	if (argv[i] && argv[i][0] != '-')
 	    nfiles++;
     char** input_filenames = gv_calloc(nfiles + 1, sizeof(char*));
 
     /* loop over arguments */
     nfiles = 0;
-    for (i = 1; i < argc; ) {
+    for (int i = 1; i < argc; ) {
 	arg = argv[i++];
 	if (*arg == '-') {
 	    i = doFlags(arg+1, i, argc, argv, &opts);
@@ -412,7 +406,7 @@ static options scanArgs(int argc, char **argv) {
 	    opts.state = -1;
 	} else {
 	    opts.program = input_filenames[0];
-	    for (i = 1; i <= nfiles; i++)
+	    for (size_t i = 1; i <= nfiles; i++)
 		input_filenames[i-1] = input_filenames[i];
 	    nfiles--;
 	}
@@ -426,7 +420,7 @@ static options scanArgs(int argc, char **argv) {
 	opts.inFiles = input_filenames;
 
     if (!opts.outFile)
-	opts.outFile = sfstdout;
+	opts.outFile = stdout;
 
   opts_done:
     if (opts.state <= 0) {
@@ -440,12 +434,11 @@ static options scanArgs(int argc, char **argv) {
 
 static Agobj_t* evalEdge(Gpr_t * state, Expr_t* prog, comp_block * xprog, Agedge_t * e)
 {
-    int i;
     case_stmt *cs;
     bool okay;
 
     state->curobj = (Agobj_t *) e;
-    for (i = 0; i < xprog->n_estmts; i++) {
+    for (size_t i = 0; i < xprog->n_estmts; i++) {
 	cs = xprog->edge_stmts + i;
 	if (cs->guard)
 	    okay = exeval(prog, cs->guard, state).integer != 0;
@@ -455,7 +448,7 @@ static Agobj_t* evalEdge(Gpr_t * state, Expr_t* prog, comp_block * xprog, Agedge
 	    if (cs->action)
 		exeval(prog, cs->action, state);
 	    else
-		agsubedge(state->target, e, TRUE);
+		agsubedge(state->target, e, 1);
 	}
     }
     return state->curobj;
@@ -463,12 +456,11 @@ static Agobj_t* evalEdge(Gpr_t * state, Expr_t* prog, comp_block * xprog, Agedge
 
 static Agobj_t* evalNode(Gpr_t * state, Expr_t* prog, comp_block * xprog, Agnode_t * n)
 {
-    int i;
     case_stmt *cs;
     bool okay;
 
     state->curobj = (Agobj_t *) n;
-    for (i = 0; i < xprog->n_nstmts; i++) {
+    for (size_t i = 0; i < xprog->n_nstmts; i++) {
 	cs = xprog->node_stmts + i;
 	if (cs->guard)
 	    okay = exeval(prog, cs->guard, state).integer != 0;
@@ -478,7 +470,7 @@ static Agobj_t* evalNode(Gpr_t * state, Expr_t* prog, comp_block * xprog, Agnode
 	    if (cs->action)
 		exeval(prog, cs->action, state);
 	    else
-		agsubnode(state->target, n, TRUE);
+		agsubnode(state->target, n, 1);
 	}
     }
     return (state->curobj);
@@ -525,21 +517,32 @@ typedef struct {
     unsigned char visit;
 } trav_fns;
 
+/// `agnxtout` wrapper to tweak calling convention
+static Agedge_t *agnxtout_(Agraph_t *g, Agedge_t *e, Agnode_t *ignored) {
+  (void)ignored;
+  return agnxtout(g, e);
+}
+
+/// `agnxtin` wrapper to tweak calling convention
+static Agedge_t *agnxtin_(Agraph_t *g, Agedge_t *e, Agnode_t *ignored) {
+  (void)ignored;
+  return agnxtin(g, e);
+}
+
 static trav_fns DFSfns = { agfstedge, agnxtedge, 1, 0 };
-static trav_fns FWDfns = { agfstout, (nxttedgefn_t) agnxtout, 0, 0 };
-static trav_fns REVfns = { agfstin, (nxttedgefn_t) agnxtin, 0, 0 };
+static trav_fns FWDfns = { agfstout, agnxtout_, 0, 0 };
+static trav_fns REVfns = { agfstin, agnxtin_, 0, 0 };
 
 static void travBFS(Gpr_t * state, Expr_t* prog, comp_block * xprog)
 {
     nodestream nodes;
-    queue *q;
+    queue_t q = {0};
     ndata *nd;
     Agnode_t *n;
     Agedge_t *cure;
     Agedge_t *nxte;
     Agraph_t *g = state->curgraph;
 
-    q = mkQueue();
     nodes.oldroot = 0;
     nodes.prev = 0;
     while ((n = nextNode(state, &nodes))) {
@@ -547,8 +550,8 @@ static void travBFS(Gpr_t * state, Expr_t* prog, comp_block * xprog)
 	if (MARKED(nd))
 	    continue;
 	PUSH(nd, 0);
-	push(q, n);
-	while ((n = pull(q))) {
+	queue_push(&q, n);
+	while ((n = queue_pop(&q))) {
 	    nd = nData(n);
 	    MARK(nd);
 	    POP(nd);
@@ -561,20 +564,22 @@ static void travBFS(Gpr_t * state, Expr_t* prog, comp_block * xprog)
 		    continue;
 		if (!evalEdge(state, prog, xprog, cure)) continue;
 		if (!ONSTACK(nd)) {
-		    push(q, cure->node);
+		    queue_push(&q, cure->node);
 		    PUSH(nd,cure);
 		}
 	    }
 	}
     }
     state->tvedge = 0;
-    freeQ(q);
+    queue_free(&q);
 }
 
+DEFINE_LIST(edge_stack, Agedge_t *)
+
 static void travDFS(Gpr_t * state, Expr_t* prog, comp_block * xprog, trav_fns * fns)
 {
     Agnode_t *n;
-    gv_stack_t stk = {0};
+    edge_stack_t stk = {0};
     Agnode_t *curn;
     Agedge_t *cure;
     Agedge_t *entry;
@@ -620,7 +625,7 @@ static void travDFS(Gpr_t * state, Expr_t* prog, comp_block * xprog, trav_fns *
 			evalEdge(state, prog, xprog, cure);
 		} else {
 		    evalEdge(state, prog, xprog, cure);
-		    stack_push(&stk, entry);
+		    edge_stack_push_back(&stk, entry);
 		    state->tvedge = entry = cure;
 		    curn = cure->node;
 		    cure = 0;
@@ -635,7 +640,7 @@ static void travDFS(Gpr_t * state, Expr_t* prog, comp_block * xprog, trav_fns *
 		nd = nData(curn);
 		POP(nd);
 		cure = entry;
-		entry = stack_is_empty(&stk) ? NULL : stack_pop(&stk);
+		entry = edge_stack_is_empty(&stk) ? NULL : edge_stack_pop_back(&stk);
 		if (entry == &(seed.out))
 		    state->tvedge = 0;
 		else
@@ -648,7 +653,7 @@ static void travDFS(Gpr_t * state, Expr_t* prog, comp_block * xprog, trav_fns *
 	}
     }
     state->tvedge = 0;
-    stack_reset(&stk);
+    edge_stack_free(&stk);
 }
 
 static void travNodes(Gpr_t * state, Expr_t* prog, comp_block * xprog)
@@ -713,10 +718,9 @@ static void doCleanup (Agraph_t* g)
 }
 
 /* traverse:
- * return 1 if traversal requires cleanup
+ * return true if traversal requires cleanup
  */
-static int traverse(Gpr_t * state, Expr_t* prog, comp_block * bp, int cleanup)
-{
+static bool traverse(Gpr_t *state, Expr_t *prog, comp_block *bp, bool cleanup) {
     if (!state->target) {
 	char *target;
 	agxbuf tmp = {0};
@@ -746,61 +750,61 @@ static int traverse(Gpr_t * state, Expr_t* prog, comp_block * bp, int cleanup)
     case TV_bfs:
 	if (cleanup) doCleanup (state->curgraph);
 	travBFS(state, prog, bp);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_dfs:
 	if (cleanup) doCleanup (state->curgraph);
 	DFSfns.visit = PRE_VISIT;
 	travDFS(state, prog, bp, &DFSfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_fwd:
 	if (cleanup) doCleanup (state->curgraph);
 	FWDfns.visit = PRE_VISIT;
 	travDFS(state, prog, bp, &FWDfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_rev:
 	if (cleanup) doCleanup (state->curgraph);
 	REVfns.visit = PRE_VISIT;
 	travDFS(state, prog, bp, &REVfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_postdfs:
 	if (cleanup) doCleanup (state->curgraph);
 	DFSfns.visit = POST_VISIT;
 	travDFS(state, prog, bp, &DFSfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_postfwd:
 	if (cleanup) doCleanup (state->curgraph);
 	FWDfns.visit = POST_VISIT;
 	travDFS(state, prog, bp, &FWDfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_postrev:
 	if (cleanup) doCleanup (state->curgraph);
 	REVfns.visit = POST_VISIT;
 	travDFS(state, prog, bp, &REVfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_prepostdfs:
 	if (cleanup) doCleanup (state->curgraph);
 	DFSfns.visit = POST_VISIT | PRE_VISIT;
 	travDFS(state, prog, bp, &DFSfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_prepostfwd:
 	if (cleanup) doCleanup (state->curgraph);
 	FWDfns.visit = POST_VISIT | PRE_VISIT;
 	travDFS(state, prog, bp, &FWDfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_prepostrev:
 	if (cleanup) doCleanup (state->curgraph);
 	REVfns.visit = POST_VISIT | PRE_VISIT;
 	travDFS(state, prog, bp, &REVfns);
-	cleanup = 1;
+	cleanup = true;
 	break;
     case TV_ne:
 	travNodes(state, prog, bp);
@@ -810,6 +814,8 @@ static int traverse(Gpr_t * state, Expr_t* prog, comp_block * bp, int cleanup)
 	travEdges(state, prog, bp);
 	travNodes(state, prog, bp);
 	break;
+    default:
+	UNREACHABLE();
     }
     return cleanup;
 }
@@ -826,8 +832,8 @@ addOutputGraph (Gpr_t* state, gvpropts* uopts)
     if ((agroot(g) == state->curgraph) && !uopts->ingraphs)
 	g = (Agraph_t*)cloneO (0, (Agobj_t *)g);
 
-    uopts->outgraphs = gv_recalloc(uopts->outgraphs, (size_t)uopts->n_outgraphs,
-                                   (size_t)uopts->n_outgraphs + 1,
+    uopts->outgraphs = gv_recalloc(uopts->outgraphs, uopts->n_outgraphs,
+                                   uopts->n_outgraphs + 1,
                                    sizeof(Agraph_t*));
     uopts->n_outgraphs++;
     uopts->outgraphs[uopts->n_outgraphs-1] = g;
@@ -844,34 +850,11 @@ static void chkClose(Agraph_t * g)
 	agclose(g);
 }
 
-static void *ing_open(char *f)
-{
-    return sfopen(f, "r");
-}
-
 static Agraph_t *ing_read(void *fp)
 {
     return readG(fp);
 }
 
-static int ing_close(void *fp)
-{
-    return sfclose(fp);
-}
-
-static ingdisc ingDisc = { ing_open, ing_read, ing_close, 0 };
-
-static void
-setDisc (Sfio_t* sp, Sfdisc_t* dp, gvprwr fn)
-{
-    dp->readf = 0;
-    dp->writef = (Sfwrite_f)fn;
-    dp->seekf = 0;
-    dp->exceptf = 0;
-    dp->disc = 0;
-    dp = sfdisc (sp, dp);
-}
-
 static jmp_buf jbuf;
 
 /* gvexitf:
@@ -887,9 +870,8 @@ gvexitf (Expr_t *handle, Exdisc_t *discipline, int v)
     longjmp (jbuf, v);
 }
 
-static int 
-gverrorf (Expr_t *handle, Exdisc_t *discipline, int level, const char *fmt, ...)
-{
+static void gverrorf(Expr_t *handle, Exdisc_t *discipline, int level,
+                     const char *fmt, ...) {
     va_list ap;
 
     va_start(ap, fmt);
@@ -904,8 +886,6 @@ gverrorf (Expr_t *handle, Exdisc_t *discipline, int level, const char *fmt, ...)
 	else if (state->flags & GV_USE_JUMP)
 	    longjmp (jbuf, 1);
     }
-
-    return 0;
 }
 
 /// collective managed state used in \p gvpr_core
@@ -929,18 +909,10 @@ typedef struct {
  */
 static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
                      gvpr_state_t *gs) {
-    Sfdisc_t errdisc;
-    Sfdisc_t outdisc;
     gpr_info info;
     int rv = 0;
-    int cleanup, i, incoreGraphs;
 
     setErrorErrors (0);
-    ingDisc.dflt = sfstdin;
-    if (uopts) {
-	if (uopts->out) setDisc (sfstdout, &outdisc, uopts->out);
-	if (uopts->err) setDisc (sfstderr, &errdisc, uopts->err);
-    }
 
     gs->opts = scanArgs(argc, argv);
     if (gs->opts.state <= 0) {
@@ -976,7 +948,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 	return 1;
     }
 
-    initGPRState(gs->state, gs->xprog->prog->vm);
+    initGPRState(gs->state);
     
     if ((uopts->flags & GV_USE_OUTGRAPH)) {
 	uopts->outgraphs = 0;
@@ -990,10 +962,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 	}
     }
 
-    if (uopts && uopts->ingraphs)
-	incoreGraphs = 1;
-    else
-	incoreGraphs = 0;
+    bool incoreGraphs = uopts && uopts->ingraphs;
 
     if (gs->opts.verbose)
 	fprintf(stderr, "Parse/compile/init: %.2f secs.\n", gvelapsed_sec());
@@ -1004,9 +973,9 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     /* if program is not null */
     if (usesGraph(gs->xprog)) {
 	if (uopts && uopts->ingraphs)
-	    gs->ing = newIngGraphs(0, uopts->ingraphs, &ingDisc);
+	    gs->ing = newIngGraphs(0, uopts->ingraphs, ing_read);
 	else
-	    gs->ing = newIng(0, gs->opts.inFiles, &ingDisc);
+	    gs->ing = newIng(0, gs->opts.inFiles, ing_read);
 	
 	if (gs->opts.verbose) gvstart_timer ();
 	Agraph_t* nextg = NULL;
@@ -1016,9 +985,9 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 	    gs->state->infname = fileName(gs->ing);
 	    if (gs->opts.readAhead)
 		nextg = gs->state->nextgraph = nextGraph(gs->ing);
-	    cleanup = 0;
+	    bool cleanup = false;
 
-	    for (i = 0; i < gs->xprog->n_blocks; i++) {
+	    for (size_t i = 0; i < gs->xprog->n_blocks; i++) {
 		comp_block* bp = gs->xprog->blocks + i;
 
 		/* begin graph */
@@ -1055,7 +1024,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 		if (uopts && (uopts->flags & GV_USE_OUTGRAPH))
 		    addOutputGraph(gs->state, uopts);
 		else
-		    sfioWrite(gs->state->outgraph, gs->opts.outFile, gs->state->dfltIO);
+		    sfioWrite(gs->state->outgraph, gs->opts.outFile);
 	    }
 
 	    if (!incoreGraphs)
@@ -1089,7 +1058,7 @@ int gvpr(int argc, char *argv[], gvpropts *uopts) {
   gvpr_state_t gvpr_state = {0};
 
   // initialize opts to something that makes freeOpts() a no-op if we fail early
-  gvpr_state.opts.outFile = sfstdout;
+  gvpr_state.opts.outFile = stdout;
 
   int rv = gvpr_core(argc, argv, uopts, &gvpr_state);
 
@@ -1102,15 +1071,6 @@ int gvpr(int argc, char *argv[], gvpropts *uopts) {
   }
   freeOpts(gvpr_state.opts);
 
-  if (uopts != NULL) {
-    if (uopts->out != NULL) {
-      sfdisc(sfstdout, 0);
-    }
-    if (uopts->err != NULL) {
-      sfdisc(sfstderr, 0);
-    }
-  }
-
   return rv;
 }
 
