@@ -10,7 +10,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -22,8 +22,6 @@
 #ifndef _WIN32
 #include <unistd.h>
 #endif
-#include <ctype.h>
-
 #include <neatogen/neato.h>
 #include <pack/pack.h>
 #include <neatogen/stress.h>
@@ -32,12 +30,21 @@
 #endif
 #include <neatogen/kkutils.h>
 #include <common/pointset.h>
+#include <common/render.h>
+#include <common/utils.h>
 #include <neatogen/sgd.h>
+#include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
 #include <cgraph/bitarray.h>
+#include <cgraph/cgraph.h>
+#include <cgraph/gv_ctype.h>
+#include <cgraph/prisize_t.h>
 #include <cgraph/startswith.h>
 #include <cgraph/strcasecmp.h>
+#include <float.h>
 #include <stdbool.h>
+#include <stddef.h>
+#include <util/streq.h>
 
 #ifndef HAVE_SRAND48
 #define srand48 srand
@@ -53,7 +60,7 @@ void neato_init_node(node_t * n)
 {
     agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);	//node custom data
     common_init_node(n);
-    ND_pos(n) = N_NEW(GD_ndim(agraphof(n)), double);
+    ND_pos(n) = gv_calloc(GD_ndim(agraphof(n)), sizeof(double));
     gv_nodesize(n, GD_flip(agraphof(n)));
 }
 
@@ -64,14 +71,13 @@ static void neato_init_edge(edge_t * e)
     ED_factor(e) = late_double(e, E_weight, 1.0, 1.0);
 }
 
-int user_pos(attrsym_t * posptr, attrsym_t * pinptr, node_t * np, int nG)
-{
+bool user_pos(attrsym_t *posptr, attrsym_t *pinptr, node_t *np, int nG) {
     double *pvec;
     char *p, c;
     double z;
 
     if (posptr == NULL)
-	return FALSE;
+	return false;
     pvec = ND_pos(np);
     p = agxget(np, posptr);
     if (p[0]) {
@@ -87,7 +93,7 @@ int user_pos(attrsym_t * posptr, attrsym_t * pinptr, node_t * np, int nG)
 		jitter_d(np, nG, 3);
 	    if (c == '!' || (pinptr && mapbool(agxget(np, pinptr))))
 		ND_pinned(np) = P_PIN;
-	    return TRUE;
+	    return true;
 	}
 	else if (sscanf(p, "%lf,%lf%c", pvec, pvec + 1, &c) >= 2) {
 	    ND_pinned(np) = P_SET;
@@ -110,12 +116,12 @@ int user_pos(attrsym_t * posptr, attrsym_t * pinptr, node_t * np, int nG)
 	    }
 	    if (c == '!' || (pinptr && mapbool(agxget(np, pinptr))))
 		ND_pinned(np) = P_PIN;
-	    return TRUE;
+	    return true;
 	} else
-	    agerr(AGERR, "node %s, position %s, expected two doubles\n",
+	    agerrorf("node %s, position %s, expected two doubles\n",
 		  agnameof(np), p);
     }
-    return FALSE;
+    return false;
 }
 
 static void neato_init_node_edge(graph_t * g)
@@ -142,8 +148,8 @@ static void neato_cleanup_graph(graph_t * g)
 {
     if (Nop || Pack < 0) {
 	free_scan_graph(g);
-	free(GD_clust(g));
     }
+    free(GD_clust(g));
 }
 
 void neato_cleanup(graph_t * g)
@@ -160,20 +166,19 @@ void neato_cleanup(graph_t * g)
     neato_cleanup_graph(g);
 }
 
-static int numFields(unsigned char *pos)
-{
+static int numFields(const char *pos) {
     int cnt = 0;
-    unsigned char c;
+    char c;
 
     do {
-	while (isspace(*pos))
+	while (gv_isspace(*pos))
 	    pos++;		/* skip white space */
 	if ((c = *pos)) { /* skip token */
 	    cnt++;
-	    while ((c = *pos) && !isspace(c) && c != ';')
+	    while ((c = *pos) && !gv_isspace(c) && c != ';')
 		pos++;
 	}
-    } while (isspace(c));
+    } while (gv_isspace(c));
     return cnt;
 }
 
@@ -189,33 +194,32 @@ static void set_label(void* obj, textlabel_t * l, char *name)
 }
 
 #ifdef IPSEPCOLA
-static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
-{
+static cluster_data cluster_map(graph_t *mastergraph, graph_t *g) {
     graph_t *subg;
     node_t *n;
      /* array of arrays of node indices in each cluster */
     int **cs,*cn;
     int i,j,nclusters=0;
     bitarray_t assigned = bitarray_new(agnnodes(g));
-    cluster_data *cdata = gv_alloc(sizeof(cluster_data));
+    cluster_data cdata = {0};
 
-    cdata->ntoplevel = agnnodes(g);
+    cdata.ntoplevel = agnnodes(g);
     for (subg = agfstsubg(mastergraph); subg; subg = agnxtsubg(subg)) {
-        if (!strncmp(agnameof(subg), "cluster", 7)) {
+        if (is_a_cluster(subg)) {
             nclusters++;
         }
     }
-    cdata->nvars=0;
-    cdata->nclusters = nclusters;
-    cs = cdata->clusters = gv_calloc(nclusters, sizeof(int*));
-    cn = cdata->clustersizes = gv_calloc(nclusters, sizeof(int));
+    cdata.nvars=0;
+    cdata.nclusters = nclusters;
+    cs = cdata.clusters = gv_calloc(nclusters, sizeof(int*));
+    cn = cdata.clustersizes = gv_calloc(nclusters, sizeof(int));
     for (subg = agfstsubg(mastergraph); subg; subg = agnxtsubg(subg)) {
         /* clusters are processed by separate calls to ordered_edges */
-        if (!strncmp(agnameof(subg), "cluster", 7)) {
+        if (is_a_cluster(subg)) {
             int *c;
 
             *cn = agnnodes(subg);
-            cdata->nvars += *cn;
+            cdata.nvars += *cn;
             c = *cs++ = gv_calloc(*cn++, sizeof(int));
             for (n = agfstnode(subg); n; n = agnxtnode(subg, n)) {
                 node_t *gn;
@@ -226,31 +230,30 @@ static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
                 }
                 *c++=ind;
                 bitarray_set(&assigned, ind, true);
-                cdata->ntoplevel--;
+                cdata.ntoplevel--;
             }
         }
     }
-    cdata->bb = gv_calloc(cdata->nclusters, sizeof(boxf));
-    cdata->toplevel = gv_calloc(cdata->ntoplevel, sizeof(int));
+    cdata.bb = gv_calloc(cdata.nclusters, sizeof(boxf));
+    cdata.toplevel = gv_calloc(cdata.ntoplevel, sizeof(int));
     for(i=j=0;i<agnnodes(g);i++) {
         if(!bitarray_get(assigned, i)) {
-            cdata->toplevel[j++]=i;
+            cdata.toplevel[j++] = i;
         }
     }
-    assert(cdata->ntoplevel==agnnodes(g)-cdata->nvars);
+    assert(cdata.ntoplevel == agnnodes(g) - cdata.nvars);
     bitarray_reset(&assigned);
     return cdata;
 }
 
-static void freeClusterData(cluster_data *c) {
-    if(c->nclusters>0) {
-        free(c->clusters[0]);
-        free(c->clusters);
-        free(c->clustersizes);
-        free(c->toplevel);
-        free(c->bb);
+static void freeClusterData(cluster_data c) {
+    if (c.nclusters > 0) {
+        free(c.clusters[0]);
+        free(c.clusters);
+        free(c.clustersizes);
+        free(c.toplevel);
+        free(c.bb);
     }
-    free(c);
 }
 #endif
 
@@ -263,7 +266,6 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 {
     char *pos;
     int i, n, npts, nc;
-    pointf *ps = 0;
     pointf *pp;
     double x, y;
     int sflag = 0, eflag = 0;
@@ -297,24 +299,24 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	    ep.y = y;
 	}
 
-	npts = numFields((unsigned char *) pos);	/* count potential points */
+	npts = numFields(pos); // count potential points
 	n = npts;
 	if (n < 4 || n % 3 != 1) {
 	    gv_free_splines(e);
 	    if (!warned) {
 		warned = true;
-		agerr(AGWARN, "pos attribute for edge (%s,%s) doesn't have 3n+1 points\n", agnameof(agtail(e)), agnameof(aghead(e)));
+		agwarningf("pos attribute for edge (%s,%s) doesn't have 3n+1 points\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    }
 	    return 0;
 	}
-	ps = ALLOC(n, 0, pointf);
+	pointf *ps = gv_calloc(n, sizeof(pointf));
 	pp = ps;
 	while (n) {
 	    i = sscanf(pos, "%lf,%lf%n", &x, &y, &nc);
 	    if (i < 2) {
 		if (!warned) {
 		    warned = true;
-		    agerr(AGWARN, "syntax error in pos attribute for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+		    agwarningf("syntax error in pos attribute for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 		}
 		free(ps);
 		gv_free_splines(e);
@@ -326,14 +328,15 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	    pp++;
 	    n--;
 	}
- 	while (isspace((int)*pos)) pos++;
+ 	while (gv_isspace(*pos)) pos++;
 	if (*pos == '\0')
 	    more = 0;
 	else
 	    pos++;
 
 	/* parsed successfully; create spline */
-	newspl = new_spline(e, npts);
+	assert(npts >= 0);
+	newspl = new_spline(e, (size_t)npts);
 	if (sflag) {
 	    newspl->sflag = stype;
 	    newspl->sp = sp;
@@ -476,7 +479,7 @@ dfs(Agraph_t * subg, Agraph_t * parentg, attrsym_t * G_lp, attrsym_t * G_bb)
 {
     boxf bb;
 
-    if (!strncmp(agnameof(subg), "cluster", 7) && chkBB(subg, G_bb, &bb)) {
+    if (is_a_cluster(subg) && chkBB(subg, G_bb, &bb)) {
 	agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	GD_bb(subg) = bb;
 	add_cluster(parentg, subg);
@@ -540,7 +543,7 @@ int init_nop(Agraph_t * g, int adjust)
     attrsym_t *G_bb = agfindgraphattr(g, "bb");
     int didAdjust = 0;  /* Have nodes been moved? */
     int haveBackground;
-    bool translate = !mapBool(agget(g, "notranslate"), false);
+    bool translate = !mapbool(agget(g, "notranslate"));
 
     /* If G_bb not defined, define it */
     if (!G_bb)
@@ -548,8 +551,8 @@ int init_nop(Agraph_t * g, int adjust)
 
     scan_graph(g);		/* mainly to set up GD_neato_nlist */
     for (i = 0; (np = GD_neato_nlist(g)[i]); i++) {
-	if (!hasPos(np) && strncmp(agnameof(np), "cluster", 7)) {
-	    agerr(AGERR, "node %s in graph %s has no position\n",
+	if (!hasPos(np) && !startswith(agnameof(np), "cluster")) {
+	    agerrorf("node %s in graph %s has no position\n",
 		  agnameof(np), agnameof(g));
 	    return -1;
 	}
@@ -640,13 +643,13 @@ static int neatoModel(graph_t * g)
 	if (agattr(g, AGEDGE, "len", 0))
 	    return MODEL_MDS;
 	else {
-	    agerr(AGWARN,
+	    agwarningf(
 	        "edges in graph %s have no len attribute. Hence, the mds model\n", agnameof(g));
 	    agerr(AGPREV, "is inappropriate. Reverting to the shortest path model.\n");
 	    return MODEL_SHORTPATH;
 	}
     }
-    agerr(AGWARN,
+    agwarningf(
 	  "Unknown value %s for attribute \"model\" in graph %s - ignored\n",
 	  p, agnameof(g));
     return MODEL_SHORTPATH;
@@ -676,7 +679,7 @@ static int neatoMode(graph_t * g)
 #endif
 #endif
 	else
-	    agerr(AGWARN,
+	    agwarningf(
 		  "Illegal value %s for attribute \"mode\" in graph %s - ignored\n",
 		  str, agnameof(g));
     }
@@ -709,21 +712,22 @@ static void
 dfsCycle (vtx_data* graph, int i,int mode, node_t* nodes[])
 {
     node_t *np, *hp;
-    int j, e, f;
+    int j;
     /* if mode is IPSEP make it an in-edge
      * at both ends, so that an edge constraint won't be generated!
      */
     double x = mode==MODE_IPSEP?-1.0:1.0;
 
     np = nodes[i];
-    ND_mark(np) = TRUE;
+    ND_mark(np) = true;
     ND_onstack(np) = true;
-    for (e = 1; e < graph[i].nedges; e++) {
+    for (size_t e = 1; e < graph[i].nedges; e++) {
 	if (graph[i].edists[e] == 1.0) continue;  /* in edge */
 	j = graph[i].edges[e];
 	hp = nodes[j];
 	if (ND_onstack(hp)) {  /* back edge: reverse it */
             graph[i].edists[e] = x;
+            size_t f;
             for (f = 1; f < graph[j].nedges && graph[j].edges[f] != i; f++) ;
             assert (f < graph[j].nedges);
             graph[j].edists[f] = -1.0;
@@ -745,7 +749,7 @@ acyclic (vtx_data* graph, int nv, int mode, node_t* nodes[])
 
     for (i = 0; i < nv; i++) {
 	np = nodes[i];
-	ND_mark(np) = FALSE;
+	ND_mark(np) = false;
 	ND_onstack(np) = false;
     }
     for (i = 0; i < nv; i++) {
@@ -785,7 +789,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
     float *edists = NULL;
 #endif
     PointMap *ps = newPM();
-    int i, i_nedges, idx;
+    int i, idx;
 
     /* lengths and weights unused in reweight model */
     bool haveLen = false;
@@ -832,7 +836,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 	else
 	    graph[i].edists = NULL;
 #endif
-	i_nedges = 1;		/* one for the self */
+	size_t i_nedges = 1; // one for the self
 
 	for (ep = agfstedge(g, np); ep; ep = agnxtedge(g, ep, np)) {
 	    if (aghead(ep) == agtail(ep))
@@ -842,8 +846,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 		if (haveWt)
 		    graph[i].eweights[idx] += ED_factor(ep);
 		if (haveLen) {
-		    int curlen = graph[i].ewgts[idx];
-		    graph[i].ewgts[idx] = MAX(ED_dist(ep), curlen);
+		    graph[i].ewgts[idx] = fmax(graph[i].ewgts[idx], ED_dist(ep));
 		}
 	    } else {
 		node_t *vp = agtail(ep) == np ? aghead(ep) : agtail(ep);
@@ -893,7 +896,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 	    eweights = gv_recalloc(graph[0].eweights, edges_size, 2 * ne + nv, sizeof(float));
 
 	for (i = 0; i < nv; i++) {
-	    int sz = graph[i].nedges;
+	    const size_t sz = graph[i].nedges;
 	    graph[i].edges = edges;
 	    edges += sz;
 	    if (haveLen) {
@@ -950,39 +953,40 @@ static void initRegular(graph_t * G, int nG)
 int
 setSeed (graph_t * G, int dflt, long* seedp)
 {
-    char smallbuf[32];
     char *p = agget(G, "start");
     int init = dflt;
 
     if (!p || *p == '\0') return dflt;
-    if (isalpha(*(unsigned char *)p)) {
-	if (!strncmp(p, SMART, SLEN(SMART))) {
+    if (gv_isalpha(*p)) {
+	if (startswith(p, SMART)) {
 	    init = INIT_SELF;
 	    p += SLEN(SMART);
-	} else if (!strncmp(p, REGULAR, SLEN(REGULAR))) {
+	} else if (startswith(p, REGULAR)) {
 	    init = INIT_REGULAR;
 	    p += SLEN(REGULAR);
-	} else if (!strncmp(p, RANDOM, SLEN(RANDOM))) {
+	} else if (startswith(p, RANDOM)) {
 	    init = INIT_RANDOM;
 	    p += SLEN(RANDOM);
 	}
 	else init = dflt;
     }
-    else if (isdigit(*(unsigned char *)p)) {
+    else if (gv_isdigit(*p)) {
 	init = INIT_RANDOM;
     }
 
     if (init == INIT_RANDOM) {
 	long seed;
 	/* Check for seed value */
-	if (!isdigit(*(unsigned char *)p) || sscanf(p, "%ld", &seed) < 1) {
+	if (!gv_isdigit(*p) || sscanf(p, "%ld", &seed) < 1) {
 #if defined(_WIN32)
 	    seed = (unsigned) time(NULL);
 #else
 	    seed = (unsigned) getpid() ^ (unsigned) time(NULL);
 #endif
-	    snprintf(smallbuf, sizeof(smallbuf), "%ld", seed);
-	    agset(G, "start", smallbuf);
+	    agxbuf buf = {0};
+	    agxbprint(&buf, "%ld", seed);
+	    agset(G, "start", agxbuse(&buf));
+	    agxbfree(&buf);
 	}
 	*seedp = seed;
     }
@@ -999,7 +1003,7 @@ static int checkExp (graph_t * G)
 {
     int exp = late_int(G, agfindgraphattr(G, exp_name), 2, 0);
     if (exp == 0 || exp > 2) {
-	agerr (AGWARN, "%s attribute value must be 1 or 2 - ignoring\n", exp_name);
+	agwarningf("%s attribute value must be 1 or 2 - ignoring\n", exp_name);
 	exp = 2;
     }
     return exp;
@@ -1023,7 +1027,7 @@ int checkStart(graph_t * G, int nG, int dflt)
     seed = 1;
     init = setSeed (G, dflt, &seed);
     if (N_pos && init != INIT_RANDOM) {
-	agerr(AGWARN, "node positions are ignored unless start=random\n");
+	agwarningf("node positions are ignored unless start=random\n");
     }
     if (init == INIT_REGULAR) initRegular(G, nG);
     srand48(seed);
@@ -1034,36 +1038,36 @@ int checkStart(graph_t * G, int nG, int dflt)
 void dumpData(graph_t * g, vtx_data * gp, int nv, int ne)
 {
     node_t *v;
-    int i, j, n;
+    int i;
 
     fprintf(stderr, "#nodes %d #edges %d\n", nv, ne);
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) {
 	fprintf(stderr, "\"%s\" %d\n", agnameof(v), ND_id(v));
     }
     for (i = 0; i < nv; i++) {
-	n = gp[i].nedges;
-	fprintf(stderr, "[%d] %d\n", i, n);
-	for (j = 0; j < n; j++) {
+	const size_t n = gp[i].nedges;
+	fprintf(stderr, "[%d] %" PRISIZE_T "\n", i, n);
+	for (size_t j = 0; j < n; j++) {
 	    fprintf(stderr, "  %3d", gp[i].edges[j]);
 	}
 	fputs("\n", stderr);
 	if (gp[i].ewgts) {
 	    fputs("  ewgts", stderr);
-	    for (j = 0; j < n; j++) {
+	    for (size_t j = 0; j < n; j++) {
 		fprintf(stderr, "  %3f", gp[i].ewgts[j]);
 	    }
 	    fputs("\n", stderr);
 	}
 	if (gp[i].eweights) {
 	    fputs("  eweights", stderr);
-	    for (j = 0; j < n; j++) {
+	    for (size_t j = 0; j < n; j++) {
 		fprintf(stderr, "  %3f", gp[i].eweights[j]);
 	    }
 	    fputs("\n", stderr);
 	}
 	if (gp[i].edists) {
 	    fputs("  edists", stderr);
-	    for (j = 0; j < n; j++) {
+	    for (size_t j = 0; j < n; j++) {
 		fprintf(stderr, "  %3f", gp[i].edists[j]);
 	    }
 	    fputs("\n", stderr);
@@ -1118,7 +1122,6 @@ void dumpOpts (ipsep_options* opp, int nv)
 static void
 majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adjust_data* am)
 {
-    double **coords;
     int ne;
     int rv = 0;
     node_t *v;
@@ -1135,8 +1138,8 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
     if (init == INIT_SELF)
 	opts |= opt_smart_init;
 
-    coords = N_GNEW(dim, double *);
-    coords[0] = N_GNEW(nv * dim, double);
+    double **coords = gv_calloc(dim, sizeof(double *));
+    coords[0] = gv_calloc(nv * dim, sizeof(double));
     for (int i = 1; i < Ndim; i++) {
 	coords[i] = coords[0] + i * nv;
     }
@@ -1155,7 +1158,7 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
 
 #ifdef DIGCOLA
     if (mode != MODE_MAJOR) {
-        double lgap = late_double(g, agfindgraphattr(g, "levelsgap"), 0.0, -MAXDOUBLE);
+        double lgap = late_double(g, agfindgraphattr(g, "levelsgap"), 0.0, -DBL_MAX);
         if (mode == MODE_HIER) {
             rv = stress_majorization_with_hierarchy(gp, nv, coords, nodes, Ndim,
                        opts, model, MaxIter, lgap);
@@ -1164,9 +1167,8 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
 	else {
             char* str;
             ipsep_options opt;
-            pointf* nsize;
-	    cluster_data *cs = cluster_map(mg,g);
-	    nsize = N_GNEW(nv, pointf);
+	    cluster_data cs = cluster_map(mg,g);
+            pointf *nsize = gv_calloc(nv, sizeof(pointf));
             opt.edge_gap = lgap;
             opt.nsize = nsize;
             opt.clusters = cs;
@@ -1247,11 +1249,10 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
 static void subset_model(Agraph_t * G, int nG)
 {
     int i, j, ne;
-    DistType **Dij;
     vtx_data *gp;
 
     gp = makeGraphData(G, nG, &ne, MODE_KK, MODEL_SUBSET, NULL);
-    Dij = compute_apsp_artifical_weights(gp, nG);
+    DistType **Dij = compute_apsp_artificial_weights(gp, nG);
     for (i = 0; i < nG; i++) {
 	for (j = 0; j < nG; j++) {
 	    GD_dist(G)[i][j] = Dij[i][j];
@@ -1292,7 +1293,7 @@ static void kkNeato(Agraph_t * g, int nG, int model)
 	subset_model(g, nG);
     } else if (model == MODEL_CIRCUIT) {
 	if (!circuit_model(g, nG)) {
-	    agerr(AGWARN,
+	    agwarningf(
 		  "graph %s is disconnected. Hence, the circuit model\n",
 		  agnameof(g));
 	    agerr(AGPREV,
@@ -1370,7 +1371,7 @@ addCluster (graph_t* g)
 {
     graph_t *subg;
     for (subg = agfstsubg(agroot(g)); subg; subg = agnxtsubg(subg)) {
-	if (!strncmp(agnameof(subg), "cluster", 7)) {
+	if (is_a_cluster(subg)) {
 	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	    add_cluster(g, subg);
 	    compute_bb(subg);
@@ -1412,7 +1413,7 @@ void neato_layout(Agraph_t * g)
 	}
 	else gv_postprocess(g, 0);
     } else {
-	bool noTranslate = mapBool(agget(g, "notranslate"), false);
+	bool noTranslate = mapbool(agget(g, "notranslate"));
 	PSinputscale = get_inputscale (g);
 	neato_init_graph(g);
 	layoutMode = neatoMode(g);
@@ -1433,17 +1434,16 @@ void neato_layout(Agraph_t * g)
 	if (Pack >= 0) {
 	    graph_t *gc;
 	    graph_t **cc;
-	    int n_cc;
-	    int i;
+	    size_t n_cc;
 	    bool pin;
 
 	    cc = pccomps(g, &n_cc, cc_pfx, &pin);
 
 	    if (n_cc > 1) {
 		bool *bp;
-		for (i = 0; i < n_cc; i++) {
+		for (size_t i = 0; i < n_cc; i++) {
 		    gc = cc[i];
-		    nodeInduce(gc);
+		    (void)graphviz_node_induce(gc, NULL);
 		    neatoLayout(g, gc, layoutMode, model, &am);
 		    removeOverlapWith(gc, &am);
 		    setEdgeType (gc, EDGETYPE_LINE);
@@ -1451,7 +1451,7 @@ void neato_layout(Agraph_t * g)
 		    else spline_edges(gc);
 		}
 		if (pin) {
-		    bp = N_NEW(n_cc, bool);
+		    bp = gv_calloc(n_cc, sizeof(bool));
 		    bp[0] = true;
 		} else
 		    bp = NULL;
@@ -1471,7 +1471,7 @@ void neato_layout(Agraph_t * g)
 	    addZ (g);
 
 	    /* cleanup and remove component subgraphs */
-	    for (i = 0; i < n_cc; i++) {
+	    for (size_t i = 0; i < n_cc; i++) {
 		gc = cc[i];
 		free_scan_graph(gc);
 		agdelrec (gc, "Agraphinfo_t");
