@@ -3,21 +3,24 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-
+#include <assert.h>
 #include "config.h"
-
+#include <cgraph/alloc.h>
 #include <cgraph/unreachable.h>
+#include <limits.h>
+#include <math.h>
 #include <neatogen/neato.h>
 #include <neatogen/adjust.h>
 #include <pathplan/pathplan.h>
 #include <pathplan/vispath.h>
 #include <neatogen/multispline.h>
 #include <stdbool.h>
+#include <stddef.h>
 
 #ifdef ORTHO
 #include <ortho/ortho.h>
@@ -39,14 +42,11 @@ static bool swap_ends_p(edge_t * e)
 static splineInfo sinfo = {.swapEnds = swap_ends_p,
                            .splineMerge = spline_merge};
 
-static void
-make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
-	      Pedge_t ** barriers, int *n_barriers)
-{
-    int i, j, k, n, b;
-    Pedge_t *bar;
+static void make_barriers(Ppoly_t **poly, int npoly, int pp, int qp,
+                          Pedge_t **barriers, size_t *n_barriers) {
+    int i, j, k;
 
-    n = 0;
+    size_t n = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
 	    continue;
@@ -54,16 +54,16 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
 	    continue;
 	n += poly[i]->pn;
     }
-    bar = N_GNEW(n, Pedge_t);
-    b = 0;
+    Pedge_t *bar = gv_calloc(n, sizeof(Pedge_t));
+    size_t b = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
 	    continue;
 	if (i == qp)
 	    continue;
-	for (j = 0; j < poly[i]->pn; j++) {
+	for (j = 0; j < (int)poly[i]->pn; j++) {
 	    k = j + 1;
-	    if (k >= poly[i]->pn)
+	    if (k >= (int)poly[i]->pn)
 		k = 0;
 	    bar[b].a = poly[i]->ps[j];
 	    bar[b].b = poly[i]->ps[k];
@@ -75,8 +75,6 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
     *n_barriers = n;
 }
 
-/* genPt:
- */
 static Ppoint_t genPt(double x, double y, pointf c)
 {
     Ppoint_t p;
@@ -86,9 +84,6 @@ static Ppoint_t genPt(double x, double y, pointf c)
     return p;
 }
 
-
-/* recPt:
- */
 static Ppoint_t recPt(double x, double y, pointf c, expand_t* m)
 {
     Ppoint_t p;
@@ -110,13 +105,12 @@ typedef struct {
     edge_t *e;
 } edgeitem;
 
-static void *newitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
-{
+static void *newitem(void *p, Dtdisc_t *disc) {
+    edgeitem *obj = p;
     edgeitem *newp;
 
-    (void)d;
     (void)disc;
-    newp = NEW(edgeitem);
+    newp = gv_alloc(sizeof(edgeitem));
     newp->id = obj->id;
     newp->e = obj->e;
     ED_count(newp->e) = 1;
@@ -124,19 +118,9 @@ static void *newitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
     return newp;
 }
 
-static void freeitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-    free(obj);
-}
-
-static int
-cmpitems(Dt_t * d, edgeinfo * key1, edgeinfo * key2, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
+static int cmpitems(void *k1, void *k2) {
+    const edgeinfo *key1 = k1;
+    const edgeinfo *key2 = k2;
     if (key1->n1 > key2->n1)
 	return 1;
     if (key1->n1 < key2->n1)
@@ -169,12 +153,9 @@ Dtdisc_t edgeItemDisc = {
     offsetof(edgeitem, id),
     sizeof(edgeinfo),
     offsetof(edgeitem, link),
-    (Dtmake_f) newitem,
-    (Dtfree_f) freeitem,
-    (Dtcompar_f) cmpitems,
-    0,
-    0,
-    0
+    newitem,
+    free,
+    cmpitems,
 };
 
 /* equivEdge:
@@ -235,7 +216,8 @@ static edge_t *equivEdge(Dt_t * map, edge_t * e)
  */
 void makeSelfArcs(edge_t * e, int stepx)
 {
-    int cnt = ED_count(e);
+    assert(ED_count(e) >= 0);
+    const size_t cnt = (size_t)ED_count(e);
 
     if (cnt == 1 || Concentrate) {
 	edge_t *edges1[1];
@@ -244,15 +226,14 @@ void makeSelfArcs(edge_t * e, int stepx)
 	if (ED_label(e))
 	    updateBB(agraphof(agtail(e)), ED_label(e));
 	makePortLabels(e);
-    } else {
-	int i;
-	edge_t **edges = N_GNEW(cnt, edge_t *);
-	for (i = 0; i < cnt; i++) {
+    } else if (cnt > 1) {
+	edge_t **edges = gv_calloc(cnt, sizeof(edge_t*));
+	for (size_t i = 0; i < cnt; i++) {
 	    edges[i] = e;
 	    e = ED_to_virt(e);
 	}
 	makeSelfEdge(edges, 0, cnt, stepx, stepx, &sinfo);
-	for (i = 0; i < cnt; i++) {
+	for (size_t i = 0; i < cnt; i++) {
 	    e = edges[i];
 	    if (ED_label(e))
 		updateBB(agraphof(agtail(e)), ED_label(e));
@@ -262,6 +243,74 @@ void makeSelfArcs(edge_t * e, int stepx)
     }
 }
 
+/** calculate the slope of the tangent of an ellipse
+ *
+ * The equation for the slope `m` of the tangent of an ellipse as a function of
+ * `x` * is given by:
+ *
+ *           bx
+ * m = ± ――――――――――
+ *          _______
+ *       a √ a²- x²
+ *
+ * or
+ *
+ * m = ± (b * x) / (a * sqrt(a * a - x * x))
+ *
+ * We know that the slope is negative in the first and third quadrant, i.e.,
+ * when the signs of x and y are the same, so we use that to select the correct
+ * slope.
+ *
+ * @param a Half the width of the ellipse, i.e., the maximum x value
+ * @param b Half the height of the ellipse, i.e., the maximum y value
+ * @param p A point on the ellipse periphery in which to calculate the slope of
+ * the tangent
+ * @return The slope of the tangent in point `p`
+ */
+static double ellipse_tangent_slope(double a, double b, pointf p) {
+  assert(p.x != a &&
+         "cannot handle ellipse tangent slope in horizontal extreme point");
+  const double sign_y = p.y >= 0 ? 1 : -1;
+  const double m = -sign_y * (b * p.x) / (a * sqrt(a * a - p.x * p.x));
+  assert(isfinite(m) && "ellipse tangent slope is infinite");
+  return m;
+}
+
+/** calculate the intersection of two lines
+ *
+ * @param l0 First line
+ * @param l1 Second line
+ * @return Intersection of the two lines
+ */
+static pointf line_intersection(linef l0, linef l1) {
+  const double x =
+      (l0.m * l0.p.x - l0.p.y - l1.m * l1.p.x + l1.p.y) / (l0.m - l1.m);
+  const double y = l0.p.y + l0.m * (x - l0.p.x);
+  return (pointf){x, y};
+}
+
+/** calculate a corner of a polygon circumscribed about an ellipse
+ *
+ * @param a Half the width of the ellipse, i.e., the maximum x value
+ * @param b Half the height of the ellipse, i.e., the maximum y value
+ * @param i Index of the polygon corner
+ * @param nsides Number of sides of the polygon
+ * @return Polygon corner at index `i`
+ */
+static pointf circumscribed_polygon_corner_about_ellipse(double a, double b,
+                                                         size_t i,
+                                                         size_t nsides) {
+  const double angle0 = 2.0 * M_PI * ((double)i - 0.5) / (double)nsides;
+  const double angle1 = 2.0 * M_PI * ((double)i + 0.5) / (double)nsides;
+  const pointf p0 = {a * cos(angle0), b * sin(angle0)};
+  const pointf p1 = {a * cos(angle1), b * sin(angle1)};
+  const double m0 = ellipse_tangent_slope(a, b, p0);
+  const double m1 = ellipse_tangent_slope(a, b, p1);
+  const linef line0 = {{p0.x, p0.y}, m0};
+  const linef line1 = {{p1.x, p1.y}, m1};
+  return line_intersection(line0, line1);
+}
+
 /* makeObstacle:
  * Given a node, return an obstacle reflecting the
  * node's geometry. pmargin specifies how much space to allow
@@ -278,32 +327,30 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 {
     Ppoly_t *obs;
     polygon_t *poly;
-    double adj = 0.0;
-    int j, sides;
+    size_t sides;
     pointf polyp;
     boxf b;
     pointf pt;
     field_t *fld;
-    int isPoly;
+    bool isPoly;
     pointf* verts = NULL;
     pointf vs[4];
     pointf p;
-    pointf margin;
+    pointf margin = {0};
 
     switch (shapeOf(n)) {
     case SH_POLY:
     case SH_POINT:
-	obs = NEW(Ppoly_t);
-	poly = (polygon_t *) ND_shape_info(n);
+	obs = gv_alloc(sizeof(Ppoly_t));
+	poly = ND_shape_info(n);
 	if (isOrtho) {
-	    isPoly = 1;
+	    isPoly = true;
 	    sides = 4;
 	    verts = vs;
-	    margin.x = margin.y = 0;
 		/* For fixedshape, we can't use the width and height, as this includes
 		 * the label. We only want to use the actual node shape.
 		 */
-	    if (poly->option & FIXEDSHAPE) {
+	    if (poly->option.fixedshape) {
 		b = polyBB (poly);
 		vs[0] = b.LL;
 		vs[1].x = b.UR.x;
@@ -312,32 +359,42 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		vs[3].x = b.LL.x;
 		vs[3].y = b.UR.y;
 	    } else {
-		p.x = -ND_lw(n);
-		p.y = -ND_ht(n)/2.0;
+		const double width = ND_lw(n) + ND_rw(n);
+		const double outline_width = INCH2PS(ND_outline_width(n));
+		// scale lw and rw proportionally to sum to outline width
+		const double outline_lw = ND_lw(n) * outline_width / width;
+		const double outline_ht = INCH2PS(ND_outline_height(n));
+		p.x = -outline_lw;
+		p.y = -outline_ht / 2.0;
 		vs[0] = p;
-		p.x = ND_lw(n);
+		p.x = outline_lw;
 		vs[1] = p;
-		p.y = ND_ht(n)/2.0;
+		p.y = outline_ht / 2.0;
 		vs[2] = p;
-		p.x = -ND_lw(n);
+		p.x = -outline_lw;
 		vs[3] = p;
 	    }
 	}
 	else if (poly->sides >= 3) {
-	    isPoly = 1;
+	    isPoly = true;
 	    sides = poly->sides;
-	    verts = poly->vertices;
-	    margin.x = pmargin->x;
-	    margin.y = pmargin->y;
+            const double penwidth = late_double(n, N_penwidth, 1.0, 0.0);
+            // possibly use extra vertices representing the outline, i.e., the
+            // outermost periphery with penwidth taken into account
+            const size_t extra_peripheries = poly->peripheries >= 1 && penwidth > 0.0 ? 1 : 0;
+            const size_t outline_periphery = poly->peripheries + extra_peripheries;
+            const size_t vertices_offset = outline_periphery >= 1 ? (outline_periphery - 1) * sides : 0;
+            verts = poly->vertices + vertices_offset;
+            margin.x = pmargin->x;
+            margin.y = pmargin->y;
 	} else {		/* ellipse */
-	    isPoly = 0;
+	    isPoly = false;
 	    sides = 8;
-	    adj = drand48() * .01;
 	}
 	obs->pn = sides;
-	obs->ps = N_NEW(sides, Ppoint_t);
+	obs->ps = gv_calloc(sides, sizeof(Ppoint_t));
 	/* assuming polys are in CCW order, and pathplan needs CW */
-	for (j = 0; j < sides; j++) {
+	for (size_t j = 0; j < sides; j++) {
 	    double xmargin = 0.0, ymargin = 0.0;
 	    if (isPoly) {
 		if (pmargin->doAdd) {
@@ -366,7 +423,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 			polyp.y = verts[j].y + ymargin;
 		    }
 		    else {
-			double h = LEN(verts[j].x,verts[j].y);
+			const double h = hypot(verts[j].x, verts[j].y);
 			polyp.x = verts[j].x * (1.0 + margin.x/h);
 			polyp.y = verts[j].y * (1.0 + margin.y/h);
 		    }
@@ -376,28 +433,23 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		    polyp.y = verts[j].y * margin.y;
 		}
 	    } else {
-		double c, s;
-		c = cos(2.0 * M_PI * j / sides + adj);
-		s = sin(2.0 * M_PI * j / sides + adj);
-		if (pmargin->doAdd) {
-		    polyp.x =  c*(ND_lw(n)+ND_rw(n)+pmargin->x) / 2.0;
-		    polyp.y =  s*(ND_ht(n)+pmargin->y) / 2.0;
-		}
-		else {
-		    polyp.x = pmargin->x * c * (ND_lw(n) + ND_rw(n)) / 2.0;
-		    polyp.y = pmargin->y * s * ND_ht(n) / 2.0;
-		}
+		const double width = INCH2PS(ND_outline_width(n));
+		const double height = INCH2PS(ND_outline_height(n));
+		margin = pmargin->doAdd ? (pointf) {pmargin->x, pmargin->y} : (pointf) {0.0, 0.0};
+		const double ellipse_a = (width + margin.x) / 2.0;
+		const double ellipse_b = (height + margin.y) / 2.0;
+		polyp = circumscribed_polygon_corner_about_ellipse(ellipse_a, ellipse_b, j, sides);
 	    }
 	    obs->ps[sides - j - 1].x = polyp.x + ND_coord(n).x;
 	    obs->ps[sides - j - 1].y = polyp.y + ND_coord(n).y;
 	}
 	break;
     case SH_RECORD:
-	fld = (field_t *) ND_shape_info(n);
+	fld = ND_shape_info(n);
 	b = fld->b;
-	obs = NEW(Ppoly_t);
+	obs = gv_alloc(sizeof(Ppoly_t));
 	obs->pn = 4;
-	obs->ps = N_NEW(4, Ppoint_t);
+	obs->ps = gv_calloc(4, sizeof(Ppoint_t));
 	/* CW order */
 	pt = ND_coord(n);
 	if (pmargin->doAdd) {
@@ -414,9 +466,9 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 	}
 	break;
     case SH_EPSF:
-	obs = NEW(Ppoly_t);
+	obs = gv_alloc(sizeof(Ppoly_t));
 	obs->pn = 4;
-	obs->ps = N_NEW(4, Ppoint_t);
+	obs->ps = gv_calloc(4, sizeof(Ppoint_t));
 	/* CW order */
 	pt = ND_coord(n);
 	if (pmargin->doAdd) {
@@ -447,7 +499,7 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
  * Assumes this info is set in ND_lim, usually from _spline_edges.
  * Returns the shortest path.
  */
-Ppolyline_t getPath(edge_t *e, vconfig_t *vconfig, int chkPts) {
+Ppolyline_t getPath(edge_t *e, vconfig_t *vconfig, bool chkPts) {
     Ppolyline_t line;
     int pp, qp;
     Ppoint_t p, q;
@@ -465,8 +517,6 @@ Ppolyline_t getPath(edge_t *e, vconfig_t *vconfig, int chkPts) {
     return line;
 }
 
-/* makePolyline:
- */
 static void makePolyline(edge_t * e) {
     Ppolyline_t spl, line = ED_path(e);
 
@@ -490,7 +540,7 @@ static void makePolyline(edge_t * e) {
 void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
     Ppolyline_t line, spline;
     Pvector_t slopes[2];
-    int i, n_barriers;
+    int i;
     int pp, qp;
     Ppoint_t p, q;
     Pedge_t *barriers;
@@ -508,11 +558,12 @@ void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
 		qp = i;
 	}
 
+    size_t n_barriers;
     make_barriers(obs, npoly, pp, qp, &barriers, &n_barriers);
     slopes[0].x = slopes[0].y = 0.0;
     slopes[1].x = slopes[1].y = 0.0;
     if (Proutespline(barriers, n_barriers, line, slopes, &spline) < 0) {
-	agerr (AGERR, "makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	agerrorf("makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	return;
     }
 
@@ -546,7 +597,6 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     Ppoly_t *obp;
     int cnt, i = 0, npoly;
     vconfig_t *vconfig = 0;
-    path *P = NULL;
     int useEdges = Nop > 1;
     int legal = 0;
 
@@ -556,7 +606,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     
     /* build configuration */
     if (edgetype >= EDGETYPE_PLINE) {
-	obs = N_NEW(agnnodes(g), Ppoly_t *);
+	obs = gv_calloc(agnnodes(g), sizeof(Ppoly_t*));
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    obp = makeObstacle(n, pmargin, edgetype == EDGETYPE_ORTHO);
 	    if (obp) {
@@ -576,9 +626,9 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 	}
 	else {
 	    if (edgetype == EDGETYPE_ORTHO)
-		agerr(AGWARN, "the bounding boxes of some nodes touch - falling back to straight line edges\n");
+		agwarningf("the bounding boxes of some nodes touch - falling back to straight line edges\n");
 	    else 
-		agerr(AGWARN, "some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
+		agwarningf("some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
 	}
     }
 
@@ -592,13 +642,13 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 	/* path-finding pass */
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-		ED_path(e) = getPath(e, vconfig, TRUE);
+		ED_path(e) = getPath(e, vconfig, true);
 	    }
 	}
     }
 #ifdef ORTHO
     else if (legal && edgetype == EDGETYPE_ORTHO) {
-	orthoEdges (g, 0);
+	orthoEdges(g, false);
 	useEdges = 1;
     }
 #endif
@@ -615,10 +665,6 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 	    } 
 	    else if (ED_count(e) == 0) continue;  /* only do representative */
 	    else if (n == head) {    /* self arc */
-		if (!P) {
-		    P = NEW(path);
-		    P->boxes = N_NEW(agnnodes(g) + 20 * 2 * 9, boxf);
-		}
 		makeSelfArcs(e, GD_nodesep(g->root));
 	    } else if (vconfig) { /* EDGETYPE_SPLINE or EDGETYPE_PLINE */
 #ifdef HAVE_GTS
@@ -661,10 +707,6 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 
     if (vconfig)
 	Pobsclose (vconfig);
-    if (P) {
-	free(P->boxes);
-	free(P);
-    }
     if (obs) {
 	for (i=0; i < npoly; i++) {
 	    free (obs[i]->ps);
@@ -762,7 +804,7 @@ void spline_edges0(graph_t *g, bool set_aspect) {
     if (et == EDGETYPE_NONE) return;
 #ifndef ORTHO
     if (et == EDGETYPE_ORTHO) {
-	agerr (AGWARN, "Orthogonal edges not yet supported\n");
+	agwarningf("Orthogonal edges not yet supported\n");
 	et = EDGETYPE_PLINE;
 	GD_flags(g->root) &= ~EDGETYPE_ORTHO;
 	GD_flags(g->root) |= EDGETYPE_PLINE;
@@ -771,8 +813,6 @@ void spline_edges0(graph_t *g, bool set_aspect) {
     spline_edges1(g, et);
 }
 
-/* shiftClusters:
- */
 static void
 shiftClusters (graph_t * g, pointf offset)
 {
@@ -815,7 +855,6 @@ void spline_edges(graph_t * g)
  */
 static void scaleEdge(edge_t * e, double xf, double yf)
 {
-    int i, j;
     pointf *pt;
     bezier *bez;
     pointf delh, delt;
@@ -826,9 +865,9 @@ static void scaleEdge(edge_t * e, double xf, double yf)
     delt.y = POINTS_PER_INCH * (ND_pos(agtail(e))[1] * (yf - 1.0));
 
     bez = ED_spl(e)->list;
-    for (i = 0; i < ED_spl(e)->size; i++) {
+    for (size_t i = 0; i < ED_spl(e)->size; i++) {
 	pt = bez->list;
-	for (j = 0; j < bez->size; j++) {
+	for (size_t j = 0; j < bez->size; j++) {
 	    if (i == 0 && j == 0) {
 		pt->x += delt.x;
 		pt->y += delt.y;
@@ -895,14 +934,13 @@ static void scaleBB(graph_t * g, double xf, double yf)
  */
 static void translateE(edge_t * e, pointf offset)
 {
-    int i, j;
     pointf *pt;
     bezier *bez;
 
     bez = ED_spl(e)->list;
-    for (i = 0; i < ED_spl(e)->size; i++) {
+    for (size_t i = 0; i < ED_spl(e)->size; i++) {
 	pt = bez->list;
-	for (j = 0; j < bez->size; j++) {
+	for (size_t j = 0; j < bez->size; j++) {
 	    pt->x -= offset.x;
 	    pt->y -= offset.y;
 	    pt++;
@@ -936,8 +974,6 @@ static void translateE(edge_t * e, pointf offset)
     }
 }
 
-/* translateG:
- */
 static void translateG(Agraph_t * g, pointf offset)
 {
     int i;
@@ -956,8 +992,6 @@ static void translateG(Agraph_t * g, pointf offset)
 	translateG(GD_clust(g)[i], offset);
 }
 
-/* neato_translate:
- */
 void neato_translate(Agraph_t * g)
 {
     node_t *n;
@@ -1006,9 +1040,7 @@ static bool _neato_set_aspect(graph_t * g)
 	}
 	/* normalize */
 	if (GD_flip(g)) {
-	    double t = GD_bb(g).UR.x;
-	    GD_bb(g).UR.x = GD_bb(g).UR.y;
-	    GD_bb(g).UR.y = t;
+	    GD_bb(g).UR = exch_xyf(GD_bb(g).UR);
 	}
 	if (GD_drawing(g)->ratio_kind == R_FILL) {
 	    /* fill is weird because both X and Y can stretch */
@@ -1032,7 +1064,7 @@ static bool _neato_set_aspect(graph_t * g)
 	    xf = (double) GD_drawing(g)->size.x / GD_bb(g).UR.x;
 	    yf = (double) GD_drawing(g)->size.y / GD_bb(g).UR.y;
 	    if (xf > 1.0 && yf > 1.0) {
-		double scale = MIN(xf, yf);
+		double scale = fmin(xf, yf);
 		xf = yf = scale;
 	    } else
 		return translated;
