@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -18,10 +18,7 @@
 #include <limits.h>
 #include <math.h>
 #include <pathplan/pathutil.h>
-
-#define ISCCW 1
-#define ISCW  2
-#define ISON  3
+#include <pathplan/tri.h>
 
 #define DQ_FRONT 1
 #define DQ_BACK  2
@@ -57,17 +54,12 @@ typedef struct deque_t {
     size_t pnlpn, fpnlpi, lpnlpi, apex;
 } deque_t;
 
-static pointnlink_t *pnls, **pnlps;
-static size_t pnln;
-static int pnll;
-
 static triangles_t tris;
 
 static Ppoint_t *ops;
 static size_t opn;
 
 static int triangulate(pointnlink_t **, int);
-static bool isdiagonal(int, int, pointnlink_t **, int);
 static int loadtriangle(pointnlink_t *, pointnlink_t *, pointnlink_t *);
 static void connecttris(size_t, size_t);
 static bool marktripath(size_t, size_t);
@@ -76,14 +68,15 @@ static void add2dq(deque_t *dq, int, pointnlink_t*);
 static void splitdq(deque_t *dq, int, size_t);
 static size_t finddqsplit(const deque_t *dq, pointnlink_t*);
 
-static int ccw(Ppoint_t *, Ppoint_t *, Ppoint_t *);
-static bool intersects(Ppoint_t *, Ppoint_t *, Ppoint_t *, Ppoint_t *);
-static bool between(Ppoint_t *, Ppoint_t *, Ppoint_t *);
 static int pointintri(size_t, Ppoint_t *);
 
-static int growpnls(size_t);
 static int growops(size_t);
 
+static Ppoint_t point_indexer(void *base, int index) {
+  pointnlink_t **b = base;
+  return *b[index]->pp;
+}
+
 /* Pshortestpath:
  * Find a shortest path contained in the polygon polyp going between the
  * points supplied in eps. The resulting polyline is stored in output.
@@ -91,35 +84,42 @@ static int growops(size_t);
  */
 int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
 {
-    int pi, minpi;
+    size_t pi, minpi;
     double minx;
     Ppoint_t p1, p2, p3;
     size_t trii, trij, ftrii, ltrii;
     int ei;
     pointnlink_t epnls[2], *lpnlp, *rpnlp, *pnlp;
     triangle_t *trip;
-#ifdef DEBUG
-    int pnli;
-#endif
 
     /* make space */
-    assert(polyp->pn >= 0);
-    if (growpnls((size_t)polyp->pn) != 0)
+    pointnlink_t *pnls = calloc(polyp->pn, sizeof(pnls[0]));
+    if (polyp->pn > 0 && pnls == NULL) {
+	prerror("cannot realloc pnls");
 	return -2;
-    pnll = 0;
+    }
+    pointnlink_t **pnlps = calloc(polyp->pn, sizeof(pnlps[0]));
+    if (polyp->pn > 0 && pnlps == NULL) {
+	prerror("cannot realloc pnlps");
+	free(pnls);
+	return -2;
+    }
+    size_t pnll = 0;
     triangles_clear(&tris);
 
-    deque_t dq = {.pnlpn = (size_t)polyp->pn * 2};
+    deque_t dq = {.pnlpn = polyp->pn * 2};
     dq.pnlps = calloc(dq.pnlpn, POINTNLINKPSIZE);
     if (dq.pnlps == NULL) {
 	prerror("cannot realloc dq.pnls");
+	free(pnlps);
+	free(pnls);
 	return -2;
     }
     dq.fpnlpi = dq.pnlpn / 2;
     dq.lpnlpi = dq.fpnlpi - 1;
 
     /* make sure polygon is CCW and load pnls array */
-    for (pi = 0, minx = HUGE_VAL, minpi = -1; pi < polyp->pn; pi++) {
+    for (pi = 0, minx = HUGE_VAL, minpi = SIZE_MAX; pi < polyp->pn; pi++) {
 	if (minx > polyp->ps[pi].x)
 	    minx = polyp->ps[pi].x, minpi = pi;
     }
@@ -127,8 +127,8 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     p1 = polyp->ps[minpi == 0 ? polyp->pn - 1 : minpi - 1];
     p3 = polyp->ps[(minpi == polyp->pn - 1) ? 0 : minpi + 1];
     if ((p1.x == p2.x && p2.x == p3.x && p3.y > p2.y) ||
-	ccw(&p1, &p2, &p3) != ISCCW) {
-	for (pi = polyp->pn - 1; pi >= 0; pi--) {
+	ccw(p1, p2, p3) != ISCCW) {
+	for (pi = polyp->pn - 1; polyp->pn > 0 && pi != SIZE_MAX; pi--) {
 	    if (pi < polyp->pn - 1
 		&& polyp->ps[pi].x == polyp->ps[pi + 1].x
 		&& polyp->ps[pi].y == polyp->ps[pi + 1].y)
@@ -151,14 +151,17 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     }
 
 #if defined(DEBUG) && DEBUG >= 1
-    fprintf(stderr, "points\n%d\n", pnll);
-    for (pnli = 0; pnli < pnll; pnli++)
+    fprintf(stderr, "points\n%" PRISIZE_T "\n", pnll);
+    for (size_t pnli = 0; pnli < pnll; pnli++)
 	fprintf(stderr, "%f %f\n", pnls[pnli].pp->x, pnls[pnli].pp->y);
 #endif
 
     /* generate list of triangles */
-    if (triangulate(pnlps, pnll)) {
+    assert(pnll <= INT_MAX);
+    if (triangulate(pnlps, (int)pnll)) {
 	free(dq.pnlps);
+	free(pnlps);
+	free(pnls);
 	return -2;
     }
 
@@ -182,6 +185,8 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     if (trii == triangles_size(&tris)) {
 	prerror("source point not in any triangle");
 	free(dq.pnlps);
+	free(pnlps);
+	free(pnls);
 	return -1;
     }
     ftrii = trii;
@@ -191,6 +196,8 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     if (trii == triangles_size(&tris)) {
 	prerror("destination point not in any triangle");
 	free(dq.pnlps);
+	free(pnlps);
+	free(pnls);
 	return -1;
     }
     ltrii = trii;
@@ -199,6 +206,8 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     if (!marktripath(ftrii, ltrii)) {
 	prerror("cannot find triangle path");
 	free(dq.pnlps);
+	free(pnlps);
+	free(pnls);
 	/* a straight line is better than failing */
 	if (growops(2) != 0)
 		return -2;
@@ -211,6 +220,8 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     /* if endpoints in same triangle, use a single line */
     if (ftrii == ltrii) {
 	free(dq.pnlps);
+	free(pnlps);
+	free(pnls);
 	if (growops(2) != 0)
 		return -2;
 	output->pn = 2;
@@ -234,15 +245,15 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
 	    if (trip->e[ei].right_index != SIZE_MAX && triangles_get(&tris, trip->e[ei].right_index).mark == 1)
 		break;
 	if (ei == 3) {		/* in last triangle */
-	    if (ccw(&eps[1], dq.pnlps[dq.fpnlpi]->pp,
-		    dq.pnlps[dq.lpnlpi]->pp) == ISCCW)
+	    if (ccw(eps[1], *dq.pnlps[dq.fpnlpi]->pp,
+		    *dq.pnlps[dq.lpnlpi]->pp) == ISCCW)
 		lpnlp = dq.pnlps[dq.lpnlpi], rpnlp = &epnls[1];
 	    else
 		lpnlp = &epnls[1], rpnlp = dq.pnlps[dq.lpnlpi];
 	} else {
 	    pnlp = trip->e[(ei + 1) % 3].pnl1p;
-	    if (ccw(trip->e[ei].pnl0p->pp, pnlp->pp,
-		    trip->e[ei].pnl1p->pp) == ISCCW)
+	    if (ccw(*trip->e[ei].pnl0p->pp, *pnlp->pp,
+		    *trip->e[ei].pnl1p->pp) == ISCCW)
 		lpnlp = trip->e[ei].pnl1p, rpnlp = trip->e[ei].pnl0p;
 	    else
 		lpnlp = trip->e[ei].pnl0p, rpnlp = trip->e[ei].pnl1p;
@@ -291,13 +302,17 @@ int Pshortestpath(Ppoly_t * polyp, Ppoint_t eps[2], Ppolyline_t * output)
     size_t i;
     for (i = 0, pnlp = &epnls[1]; pnlp; pnlp = pnlp->link)
 	i++;
-    if (growops(i) != 0)
+    if (growops(i) != 0) {
+	free(pnlps);
+	free(pnls);
 	return -2;
-    assert(i <= INT_MAX);
-    output->pn = (int)i;
+    }
+    output->pn = i;
     for (i = i - 1, pnlp = &epnls[1]; pnlp; i--, pnlp = pnlp->link)
 	ops[i] = *pnlp->pp;
     output->ps = ops;
+    free(pnlps);
+    free(pnls);
 
     return 0;
 }
@@ -312,7 +327,7 @@ static int triangulate(pointnlink_t **points, int point_count) {
 		{
 			pnlip1 = (pnli + 1) % point_count;
 			pnlip2 = (pnli + 2) % point_count;
-			if (isdiagonal(pnli, pnlip2, points, point_count))
+			if (isdiagonal(pnli, pnlip2, points, point_count, point_indexer))
 			{
 				if (loadtriangle(points[pnli], points[pnlip1], points[pnlip2]) != 0)
 					return -1;
@@ -331,35 +346,6 @@ static int triangulate(pointnlink_t **points, int point_count) {
     return 0;
 }
 
-/* check if (i, i + 2) is a diagonal */
-static bool isdiagonal(int pnli, int pnlip2, pointnlink_t **points,
-                       int point_count) {
-    int pnlip1, pnlim1, pnlj, pnljp1, res;
-
-    /* neighborhood test */
-    pnlip1 = (pnli + 1) % point_count;
-    pnlim1 = (pnli + point_count - 1) % point_count;
-    /* If P[pnli] is a convex vertex [ pnli+1 left of (pnli-1,pnli) ]. */
-    if (ccw(points[pnlim1]->pp, points[pnli]->pp, points[pnlip1]->pp) == ISCCW)
-	res = ccw(points[pnli]->pp, points[pnlip2]->pp, points[pnlim1]->pp) == ISCCW
-	   && ccw(points[pnlip2]->pp, points[pnli]->pp, points[pnlip1]->pp) == ISCCW;
-    /* Assume (pnli - 1, pnli, pnli + 1) not collinear. */
-    else
-	res = ccw(points[pnli]->pp, points[pnlip2]->pp, points[pnlip1]->pp) == ISCW;
-    if (!res)
-	return false;
-
-    /* check against all other edges */
-    for (pnlj = 0; pnlj < point_count; pnlj++) {
-	pnljp1 = (pnlj + 1) % point_count;
-	if (!(pnlj == pnli || pnljp1 == pnli || pnlj == pnlip2 || pnljp1 == pnlip2))
-	    if (intersects(points[pnli]->pp, points[pnlip2]->pp,
-			   points[pnlj]->pp, points[pnljp1]->pp))
-		return false;
-    }
-    return true;
-}
-
 static int loadtriangle(pointnlink_t * pnlap, pointnlink_t * pnlbp,
 			 pointnlink_t * pnlcp)
 {
@@ -435,87 +421,24 @@ static void splitdq(deque_t *dq, int side, size_t index) {
 
 static size_t finddqsplit(const deque_t *dq, pointnlink_t *pnlp) {
     for (size_t index = dq->fpnlpi; index < dq->apex; index++)
-	if (ccw(dq->pnlps[index + 1]->pp, dq->pnlps[index]->pp, pnlp->pp) == ISCCW)
+	if (ccw(*dq->pnlps[index + 1]->pp, *dq->pnlps[index]->pp, *pnlp->pp) == ISCCW)
 	    return index;
     for (size_t index = dq->lpnlpi; index > dq->apex; index--)
-	if (ccw(dq->pnlps[index - 1]->pp, dq->pnlps[index]->pp, pnlp->pp) == ISCW)
+	if (ccw(*dq->pnlps[index - 1]->pp, *dq->pnlps[index]->pp, *pnlp->pp) == ISCW)
 	    return index;
     return dq->apex;
 }
 
-/* ccw test: CCW, CW, or co-linear */
-static int ccw(Ppoint_t * p1p, Ppoint_t * p2p, Ppoint_t * p3p)
-{
-    double d;
-
-    d = (p1p->y - p2p->y) * (p3p->x - p2p->x) -
-	(p3p->y - p2p->y) * (p1p->x - p2p->x);
-    return d > 0 ? ISCCW : (d < 0 ? ISCW : ISON);
-}
-
-/* line to line intersection */
-static bool intersects(Ppoint_t * pap, Ppoint_t * pbp,
-		      Ppoint_t * pcp, Ppoint_t * pdp)
-{
-    int ccw1, ccw2, ccw3, ccw4;
-
-    if (ccw(pap, pbp, pcp) == ISON || ccw(pap, pbp, pdp) == ISON ||
-	ccw(pcp, pdp, pap) == ISON || ccw(pcp, pdp, pbp) == ISON) {
-	if (between(pap, pbp, pcp) || between(pap, pbp, pdp) ||
-	    between(pcp, pdp, pap) || between(pcp, pdp, pbp))
-	    return true;
-    } else {
-	ccw1 = ccw(pap, pbp, pcp) == ISCCW ? 1 : 0;
-	ccw2 = ccw(pap, pbp, pdp) == ISCCW ? 1 : 0;
-	ccw3 = ccw(pcp, pdp, pap) == ISCCW ? 1 : 0;
-	ccw4 = ccw(pcp, pdp, pbp) == ISCCW ? 1 : 0;
-	return (ccw1 ^ ccw2) && (ccw3 ^ ccw4);
-    }
-    return false;
-}
-
-/* is pbp between pap and pcp */
-static bool between(Ppoint_t * pap, Ppoint_t * pbp, Ppoint_t * pcp)
-{
-    Ppoint_t p1, p2;
-
-    p1.x = pbp->x - pap->x, p1.y = pbp->y - pap->y;
-    p2.x = pcp->x - pap->x, p2.y = pcp->y - pap->y;
-    if (ccw(pap, pbp, pcp) != ISON)
-	return false;
-    return p2.x * p1.x + p2.y * p1.y >= 0 &&
-	p2.x * p2.x + p2.y * p2.y <= p1.x * p1.x + p1.y * p1.y;
-}
-
 static int pointintri(size_t trii, Ppoint_t *pp) {
     int ei, sum;
 
     for (ei = 0, sum = 0; ei < 3; ei++)
-	if (ccw(triangles_get(&tris, trii).e[ei].pnl0p->pp,
-	        triangles_get(&tris, trii).e[ei].pnl1p->pp, pp) != ISCW)
+	if (ccw(*triangles_get(&tris, trii).e[ei].pnl0p->pp,
+	        *triangles_get(&tris, trii).e[ei].pnl1p->pp, *pp) != ISCW)
 	    sum++;
     return sum == 3 || sum == 0;
 }
 
-static int growpnls(size_t newpnln) {
-    if (newpnln <= pnln)
-	return 0;
-    pointnlink_t *new_pnls = realloc(pnls, POINTNLINKSIZE * newpnln);
-    if (new_pnls == NULL) {
-	prerror("cannot realloc pnls");
-	return -1;
-    }
-    pnls = new_pnls;
-    pointnlink_t **new_pnlps = realloc(pnlps, POINTNLINKPSIZE * newpnln);
-    if (new_pnlps == NULL) {
-	prerror("cannot realloc pnlps");
-	return -1;
-    }
-    pnlps = new_pnlps;
-    pnln = newpnln;
-    return 0;
-}
-
 static int growops(size_t newopn) {
     if (newopn <= opn)
 	return 0;
